@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2007 Zuza Software Foundation
@@ -26,19 +25,6 @@ from subprocess import call
 from lxml import etree
 
 
-# get directory of this test
-dir = os.path.dirname(os.path.abspath(__file__))
-# get top-level directory (moral equivalent of ../..)
-dir = os.path.dirname(os.path.dirname(dir))
-# load python modules from top-level
-sys.path.insert(0, dir)
-# add top-level to PYTHONPATH for subprocesses
-os.environ["PYTHONPATH"] = os.pathsep.join(sys.path)
-# add {top-level}/translate/convert to PATH for [po]o2xliff
-os.environ["PATH"] = os.pathsep.join([os.path.join(dir,
-                                                   "translate", "convert"),
-                                      os.environ["PATH"]])
-
 schema = None
 
 
