@@ -30,10 +30,10 @@
 import glob
 import os
 import shutil
-import StringIO
+import six
 import tempfile
 import time
-from subprocess import Popen, PIPE, STDOUT
+from subprocess import Popen, PIPE
 
 join = os.path.join
 
@@ -66,7 +66,7 @@ options = {'verbose': True}
 USAGE = 'Usage: %prog [options] <langs...|ALL>'
 
 
-class CommandError(StandardError):
+class CommandError(Exception):
     """Exception raised if a command does not return its expected value."""
 
     def __init__(self, cmd, status):
@@ -82,10 +82,10 @@ class CommandError(StandardError):
 
 def delfiles(pattern, path, files):
     """Delete files with names in C{files} matching glob-pattern C{glob} in the
-        directory specified by C{path}.
+    directory specified by C{path}.
 
-        This function is meant to be used with C{os.walk}
-        """
+    This function is meant to be used with C{os.walk}
+    """
     path = os.path.abspath(path)
     match_files = glob.glob(join(path, pattern))
     for f in files:
@@ -94,7 +94,6 @@ def delfiles(pattern, path, files):
 
 
 def run(cmd, expected_status=0, stdout=None, stderr=None, shell=False):
-    global options
     if options['verbose']:
         print('>>> %s $ %s' % (os.getcwd(), ' '.join(cmd)))
     p = Popen(cmd, stdout=stdout, stderr=stderr, shell=shell)
@@ -113,16 +112,16 @@ def run(cmd, expected_status=0, stdout=None, stderr=None, shell=False):
 
 def get_langs(lang_args):
     """Returns the languages to handle based on the languages specified on the
-        command-line.
+    command-line.
 
-        If "ALL" was specified, the languages are read from the Mozilla
-        product's C{shipped-locales} file. If "ZA" was specified, all South
-        African languages are selected.
-        """
+    If "ALL" was specified, the languages are read from the Mozilla product's
+    C{shipped-locales} file. If "ZA" was specified, all South African languages
+    are selected.
+    """
 
     langs = []
 
-    if isinstance(lang_args, str):
+    if isinstance(lang_args, six.string_types):
         if lang_args == 'ALL':
             lang_args = ['ALL']
         elif lang_args == 'ZA':
@@ -139,10 +138,11 @@ def get_langs(lang_args):
             # Get all available languages from the locales file
             locales_filename = join(mozilladir, targetapp,
                                     'locales', 'shipped-locales')
-            for line in open(locales_filename).readlines():
-                langcode = line.split()[0]
-                if langcode != 'en-US':
-                    langs.append(langcode)
+            with open(locales_filename, 'r') as fh:
+                for line in fh:
+                    langcode = line.split()[0]
+                    if langcode != 'en-US':
+                        langs.append(langcode)
 
         elif lang == 'ZA':
             # South African languages
@@ -224,10 +224,12 @@ def checkout(cvstag, langs):
             raise oe
 
     if mozversion < '3':
-        for f in ['en-US/browser/README.txt pot/browser/README.txt.pot',
-                  'en-US/browser/os2/README.txt pot/browser/os2/README.txt.pot',
-                  'en-US/mail/README.txt pot/mail/README.txt.pot',
-                  'en-US/mail/os2/README.txt pot/mail/os2/README.txt.pot',]:
+        for f in [
+            'en-US/browser/README.txt pot/browser/README.txt.pot',
+            'en-US/browser/os2/README.txt pot/browser/os2/README.txt.pot',
+            'en-US/mail/README.txt pot/mail/README.txt.pot',
+            'en-US/mail/os2/README.txt pot/mail/os2/README.txt.pot',
+        ]:
             run(['txt2po', '--progress=none', '-P', f])
     os.chdir(olddir)
 
@@ -336,7 +338,7 @@ def post_po2moz_hacks(lang, buildlang):
     if os.path.isfile(inst_inc_po):
         tempdir = tempfile.mkdtemp()
         tmp_po = join(tempdir, 'installer.%s.properties.po' % (lang))
-        shutil.copy2(inst_po, tmp_po)
+        shutil.copy2(inst_inc_po, tmp_po)
 
         inst_inc = join(l10ndir, 'en-US', 'mail', 'installer', 'installer.inc')
         tmp_properties = join(tempdir, 'installer.properties')
@@ -481,15 +483,17 @@ def create_diff(lang, buildlang):
 
     os.chdir(l10ndir)
     outfile = join(os.pardir, 'diff', buildlang + '-l10n.diff')
-    run(['cvs', 'diff', '--newfile', buildlang], stdout=open(outfile, 'w'))
+    with open(outfile, 'w') as fh:
+        run(['cvs', 'diff', '--newfile', buildlang], stdout=fh)
     os.chdir(olddir)
 
     os.chdir(join(podir_updated, buildlang))
     outfile = join(os.pardir, os.pardir, 'diff', buildlang + '-po.diff')
-    run(['svn', 'diff',
-         '--diff-cmd',
-         'diff -x "-u --ignore-matching-lines=^\"POT\|^\"X-Gene"'],
-         stdout=open(outfile, 'w'))
+    with open(outfile, 'w') as fh:
+        run(['svn', 'diff',
+             '--diff-cmd',
+             'diff -x "-u --ignore-matching-lines=^\"POT\|^\"X-Gene"'],
+            stdout=fh)
     os.chdir(olddir)
 
 
@@ -663,7 +667,7 @@ def main(langs=['ALL'], mozproduct='browser', mozcheckout=False, moztag='-A',
 
 
 def main_cmd_line():
-    options, langs = create_option_parser().parse_known_args()
+    args, langs = create_option_parser().parse_known_args()
 
     main(
         langs=langs,
