@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2006 Zuza Software Foundation
@@ -19,10 +18,12 @@
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Handles converting of files between formats (used by
-:mod:`translate.convert` tools)."""
+:mod:`translate.convert` tools).
+"""
 
 import os.path
-from cStringIO import StringIO
+import six
+from io import BytesIO
 
 from translate.misc import optrecurse
 
@@ -59,6 +60,15 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
                         action="store_false", default=default, help=nofuzzyhelp)
         self.passthrough.append("includefuzzy")
 
+    def add_remove_untranslated_option(self, default=False):
+        """Adds an option to remove key value from output if it is
+        untranslated.
+        """
+        self.add_option("", "--removeuntranslated", dest="remove_untranslated",
+                        default=False, action="store_true",
+                        help="remove untranslated strings from output")
+        self.passthrough.append("remove_untranslated")
+
     def add_threshold_option(self, default=None):
         """Adds an option to output only stores where translation percentage
         exceeds the threshold.
@@ -123,7 +133,7 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
         """Filters output options, processing relevant switches in options."""
         if self.usepots and options.pot:
             outputoptions = {}
-            for (inputformat, templateformat), (outputformat, convertor) in self.outputoptions.iteritems():
+            for (inputformat, templateformat), (outputformat, convertor) in six.iteritems(self.outputoptions):
                 inputformat = self.potifyformat(inputformat)
                 templateformat = self.potifyformat(templateformat)
                 outputformat = self.potifyformat(outputformat)
@@ -133,8 +143,9 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
             return self.outputoptions
 
     def setpotoption(self):
-        """Sets the ``-P``/``--pot`` option depending on input/output
-        formats etc."""
+        """Sets the ``-P``/``--pot`` option depending on input/output formats
+        etc.
+        """
         if self.usepots:
             potoption = optparse.Option(
                 "-P", "--pot",
@@ -153,15 +164,16 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
         self.define_option(timestampopt)
 
     def verifyoptions(self, options):
-        """Verifies that the options are valid (required options are
-        present, etc)."""
+        """Verifies that the options are valid (required options are present,
+        etc).
+        """
         pass
 
     def run(self, argv=None):
         """Parses the command line options and runs the conversion."""
         (options, args) = self.parse_args(argv)
-        options.inputformats = self.filterinputformats(options)
-        options.outputoptions = self.filteroutputoptions(options)
+        self.inputformats = self.filterinputformats(options)
+        self.outputoptions = self.filteroutputoptions(options)
         try:
             self.verifyoptions(options)
         except Exception as e:
@@ -173,10 +185,9 @@ class ConvertOptionParser(optrecurse.RecursiveOptionParser, object):
         if options.timestamp and _output_is_newer(fullinputpath, fulloutputpath):
             return False
 
-        return super(ConvertOptionParser,
-                    self).processfile(fileprocessor, options,
-                                      fullinputpath, fulloutputpath,
-                                      fulltemplatepath)
+        return super(ConvertOptionParser, self).processfile(
+            fileprocessor, options, fullinputpath, fulloutputpath,
+            fulltemplatepath)
 
 
 def copyinput(inputfile, outputfile, templatefile, **kwargs):
@@ -191,7 +202,7 @@ def copytemplate(inputfile, outputfile, templatefile, **kwargs):
     return True
 
 
-class Replacer:
+class Replacer(object):
     """An object that knows how to replace strings in files."""
 
     def __init__(self, searchstring, replacestring):
@@ -214,7 +225,8 @@ class Replacer:
     def searchreplacetemplate(self, inputfile, outputfile,
                               templatefile, **kwargs):
         """Copies the template file to the output file, searching and
-        replacing."""
+        replacing.
+        """
         outputfile.write(self.doreplace(templatefile.read()))
         return True
 
@@ -240,8 +252,9 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
     ``archiveformats`` maps extension to class. If the extension doesn't
     matter, it can be None.
 
-    If the extension is only valid for input/output/template, it can be
-    given as ``(extension, filepurpose)``."""
+    If the extension is only valid for input/output/template, it can be given
+    as ``(extension, filepurpose)``.
+    """
 
     def __init__(self, formats, usetemplates=False, usepots=False,
                  description=None, archiveformats=None):
@@ -253,10 +266,6 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
         ConvertOptionParser.__init__(self, formats, usetemplates, usepots,
                                      description=description)
 
-    def setarchiveoptions(self, **kwargs):
-        """Allows setting options that will always be passed to openarchive."""
-        self.archiveoptions = kwargs
-
     def isrecursive(self, fileoption, filepurpose='input'):
         """Checks if **fileoption** is a recursive file."""
         if self.isarchive(fileoption, filepurpose):
@@ -266,7 +275,7 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
 
     def isarchive(self, fileoption, filepurpose='input'):
         """Returns whether the file option is an archive file."""
-        if not isinstance(fileoption, (str, unicode)):
+        if not isinstance(fileoption, six.string_types):
             return False
         mustexist = (filepurpose != 'output')
         if mustexist and not os.path.isfile(fileoption):
@@ -303,8 +312,9 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
         return archiveclass(archivefilename, **archiveoptions)
 
     def recurseinputfiles(self, options):
-        """Recurse through archive file / directories and return files
-        to be converted."""
+        """Recurse through archive file / directories and return files to be
+        converted.
+        """
         if self.isarchive(options.input, 'input'):
             options.inputarchive = self.openarchive(options.input, 'input')
             return self.recursearchivefiles(options)
@@ -318,7 +328,7 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
             if self.isexcluded(options, inputpath):
                 continue
             top, name = os.path.split(inputpath)
-            if not self.isvalidinputname(options, name):
+            if not self.isvalidinputname(name):
                 continue
             inputfiles.append(inputpath)
         return inputfiles
@@ -343,8 +353,8 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
             if (options.recursivetemplate and
                 self.isarchive(options.template, 'template')):
                 # TODO: deal with different names in input/template archives
-                if fulltemplatepath in options.templatearchive:
-                    return options.templatearchive.openinputfile(fulltemplatepath)
+                if fulltemplatepath in self.templatearchive:
+                    return self.templatearchive.openinputfile(fulltemplatepath)
                 else:
                     self.warning("missing template file %s" % fulltemplatepath)
         return super(ArchiveConvertOptionParser, self).opentemplatefile(options, fulltemplatepath)
@@ -366,7 +376,7 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
         if templatepath is not None:
             if self.isarchive(options.template, 'template'):
                 # TODO: deal with different names in input/template archives
-                return templatepath in options.templatearchive
+                return templatepath in self.templatearchive
         return super(ArchiveConvertOptionParser, self).templateexists(options, templatepath)
 
     def getfulloutputpath(self, options, outputpath):
@@ -380,49 +390,40 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
 
     def checkoutputsubdir(self, options, subdir):
         """Checks to see if subdir under ``options.output`` needs to be
-        created, creates if neccessary."""
+        created, creates if neccessary.
+        """
         if not self.isarchive(options.output, 'output'):
             super(ArchiveConvertOptionParser, self).checkoutputsubdir(options, subdir)
 
     def openoutputfile(self, options, fulloutputpath):
         """Opens the output file."""
         if self.isarchive(options.output, 'output'):
-            outputstream = options.outputarchive.openoutputfile(fulloutputpath)
+            outputstream = self.outputarchive.openoutputfile(fulloutputpath)
             if outputstream is None:
                 self.warning("Could not find where to put %s in output "
                              "archive; writing to tmp" % fulloutputpath)
-                return StringIO()
+                return BytesIO()
             return outputstream
         else:
             return super(ArchiveConvertOptionParser, self).openoutputfile(options, fulloutputpath)
 
-    def inittemplatearchive(self, options):
-        """Opens the ``templatearchive`` if not already open."""
-        if not self.usetemplates:
-            return
-        if (options.template and
-            self.isarchive(options.template, 'template') and
-            not hasattr(options, "templatearchive")):
-            options.templatearchive = self.openarchive(options.template,
-                                                       'template')
-
-    def initoutputarchive(self, options):
-        """Creates an outputarchive if required."""
-        if options.output and self.isarchive(options.output, 'output'):
-            options.outputarchive = self.openarchive(options.output, 'output',
-                                                     mode="w")
-
     def recursiveprocess(self, options):
         """Recurse through directories and convert files."""
         if hasattr(options, "multifilestyle"):
-            self.setarchiveoptions(multifilestyle=options.multifilestyle)
+            self.archiveoptions = {'multifilestyle': options.multifilestyle}
             for filetype in ("input", "output", "template"):
                 allowoption = "allowrecursive%s" % filetype
                 if (options.multifilestyle == "onefile" and
                     getattr(options, allowoption, True)):
                     setattr(options, allowoption, False)
-        self.inittemplatearchive(options)
-        self.initoutputarchive(options)
+
+        if (self.usetemplates and options.template and
+            self.isarchive(options.template, 'template')):
+            self.templatearchive = self.openarchive(options.template, 'template')
+
+        if options.output and self.isarchive(options.output, 'output'):
+            self.outputarchive = self.openarchive(options.output, 'output',
+                                                  mode="w")
         return super(ArchiveConvertOptionParser, self).recursiveprocess(options)
 
     def processfile(self, fileprocessor, options, fullinputpath,
@@ -448,10 +449,9 @@ class ArchiveConvertOptionParser(ConvertOptionParser):
                     os.unlink(fulloutputpath)
                 return False
         else:
-            return super(ArchiveConvertOptionParser,
-                        self).processfile(fileprocessor, options,
-                                          fullinputpath, fulloutputpath,
-                                          fulltemplatepath)
+            return super(ArchiveConvertOptionParser, self).processfile(
+                fileprocessor, options, fullinputpath, fulloutputpath,
+                fulltemplatepath)
 
 
 def _output_is_newer(input_path, output_path):
@@ -480,13 +480,16 @@ def should_output_store(store, threshold):
 
     from translate.storage import statsdb
 
-    units = filter(lambda unit: unit.istranslatable(), store.units)
-    translated = filter(lambda unit: unit.istranslated(), units)
-    wordcounts = dict(map(lambda unit: (unit, statsdb.wordsinunit(unit)), units))
-    sourcewords = lambda elementlist: sum(map(lambda unit: wordcounts[unit][0], elementlist))
-    tranlated_count = sourcewords(translated)
+    units = list(filter(lambda unit: unit.istranslatable(), store.units))
+    translated = list(filter(lambda unit: unit.istranslated(), units))
+    wordcounts = dict(map(lambda unit: (unit.getid(), statsdb.wordsinunit(unit)), units))
+
+    def sourcewords(elementlist):
+        return sum(map(lambda unit: wordcounts[unit.getid()][0], elementlist))
+
+    translated_count = sourcewords(translated)
     total_count = sourcewords(units)
-    percent = tranlated_count * 100 / total_count
+    percent = translated_count * 100 / total_count
 
     return percent >= threshold
 
