@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2006 Zuza Software Foundation
@@ -28,7 +27,7 @@ for examples and usage instructions.
 from translate.storage import html, po
 
 
-class html2po:
+class html2po(object):
 
     def convertfile(self, inputfile, filename, includeuntagged=False,
                     duplicatestyle="msgctxt", keepcomments=False):
@@ -48,23 +47,20 @@ class html2po:
 def converthtml(inputfile, outputfile, templates, includeuntagged=False,
                 pot=False, duplicatestyle="msgctxt", keepcomments=False):
     """reads in stdin using fromfileclass, converts using convertorclass,
-    writes to stdout"""
+    writes to stdout
+    """
     convertor = html2po()
-    outputfilepos = outputfile.tell()
     outputstore = convertor.convertfile(inputfile, getattr(inputfile, "name",
                                                            "unknown"),
                                         includeuntagged,
                                         duplicatestyle=duplicatestyle,
                                         keepcomments=keepcomments)
-    outputfile.write(str(outputstore))
+    outputstore.serialize(outputfile)
     return 1
 
 
 def main(argv=None):
     from translate.convert import convert
-    from translate.misc import stdiotell
-    import sys
-    sys.stdout = stdiotell.StdIOWrapper(sys.stdout)
     formats = {
         "html": ("po", converthtml),
         "htm": ("po", converthtml),
