@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2008,2010-2011 Zuza Software Foundation
@@ -26,6 +25,7 @@ for examples and usage instructions.
 
 import logging
 import os
+import six
 import time
 
 from translate.convert import convert
@@ -36,7 +36,7 @@ from translate.storage import factory, oo
 logger = logging.getLogger(__name__)
 
 
-class reoo:
+class reoo(object):
 
     def __init__(self, templatefile, languages=None, timestamp=None, includefuzzy=False, long_keys=False, filteraction="exclude"):
         """construct a reoo converter for the specified languages (timestamp=0 means leave unchanged)"""
@@ -58,7 +58,7 @@ class reoo:
     def makeindex(self):
         """makes an index of the oo keys that are used in the source file"""
         self.index = {}
-        for ookey, theoo in self.o.ookeys.iteritems():
+        for ookey, theoo in six.iteritems(self.o.ookeys):
             sourcekey = oo.makekey(ookey, self.long_keys)
             self.index[sourcekey] = theoo
 
@@ -92,10 +92,10 @@ class reoo:
                                key, len(self.index))
                 try:
                     sourceunitlines = str(unit)
-                    if isinstance(sourceunitlines, unicode):
+                    if isinstance(sourceunitlines, six.text_type):
                         sourceunitlines = sourceunitlines.encode("utf-8")
                     logger.warning(sourceunitlines)
-                except:
+                except Exception:
                     logger.warning("error outputting source unit %r", str(unit))
 
     def applytranslation(self, key, subkey, theoo, unit):
@@ -122,8 +122,6 @@ class reoo:
         # If there is no translation, we don't want to add a line
         if len(unquotedstr) == 0:
             return
-        if isinstance(unquotedstr, unicode):
-            unquotedstr = unquotedstr.encode("UTF-8")
         # finally set the new definition in the oo, but not if its empty
         if len(unquotedstr) > 0:
             setattr(part2, subkey, unquotedstr)
@@ -159,20 +157,20 @@ class oocheckfilter(pofilter.pocheckfilter):
         filterresult = self.filterunit(unit)
         if filterresult:
             if filterresult != autocorrect:
-                for filtername, filtermessage in filterresult.iteritems():
+                for filtername, filtermessage in six.iteritems(filterresult):
                     location = unit.getlocations()[0].encode('utf-8')
                     if filtername in self.options.error:
                         logger.error("Error at %s::%s: %s",
                                      filename, location, filtermessage)
-                        return not filteraction in ["exclude-all", "exclude-serious"]
+                        return filteraction not in ["exclude-all", "exclude-serious"]
                     if filtername in self.options.warning or self.options.alwayswarn:
                         logger.warning("Warning at %s::%s: %s",
                                        filename, location, filtermessage)
-                        return not filteraction in ["exclude-all"]
+                        return filteraction not in ["exclude-all"]
         return True
 
 
-class oofilteroptions:
+class oofilteroptions(object):
     error = ['variables', 'xmltags', 'escapes']
     warning = ['blank']
     #To only issue warnings for tests listed in warning, change the following to False:
@@ -186,6 +184,7 @@ class oofilteroptions:
     includereview = False
     autocorrect = False
 
+
 options = oofilteroptions()
 filter = oocheckfilter(options, [checks.OpenOfficeChecker, checks.StandardUnitChecker], checks.openofficeconfig)
 
@@ -217,16 +216,17 @@ def convertoo(inputfile, outputfile, templatefile, sourcelanguage=None,
                          filteraction=filteraction)
     outputstore = convertor.convertstore(inputstore)
     # TODO: check if we need to manually delete missing items
-    outputfile.write(outputstore.__str__(skip_source, targetlanguage))
+    outputstore.serialize(outputfile, skip_source, targetlanguage)
     return True
 
 
 def main(argv=None):
     formats = {
-                ("po", "oo"): ("oo", convertoo),
-                ("xlf", "oo"): ("oo", convertoo),
-                ("po", "sdf"): ("sdf", convertoo),
-              }
+        ("po", "oo"): ("oo", convertoo),
+        ("xlf", "oo"): ("oo", convertoo),
+        ("xliff", "oo"): ("oo", convertoo),
+        ("po", "sdf"): ("sdf", convertoo),
+    }
     # always treat the input as an archive unless it is a directory
     archiveformats = {(None, "output"): oo.oomultifile, (None, "template"): oo.oomultifile}
     parser = convert.ArchiveConvertOptionParser(formats, usetemplates=True, description=__doc__, archiveformats=archiveformats)
@@ -237,9 +237,10 @@ def main(argv=None):
                       default=None,
                       help="set source language code (default en-US)",
                       metavar="LANG")
-    parser.add_option("-T", "--keeptimestamp", dest="timestamp", default=None,
-                      action="store_const", const=0,
-            help="don't change the timestamps of the strings")
+    parser.add_option(
+        "-T", "--keeptimestamp", dest="timestamp", default=None,
+        action="store_const", const=0,
+        help="don't change the timestamps of the strings")
     parser.add_option("", "--nonrecursiveoutput", dest="allowrecursiveoutput",
                       default=True, action="store_false",
                       help="don't treat the output oo as a recursive store")
