@@ -1,9 +1,7 @@
-#!/usr/bin/env python
-
 from translate.convert import moz2po, test_convert
 
 
-class TestMoz2PO:
+class TestMoz2PO(object):
     pass
 
 
