@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+from pytest import mark
+
 from translate.convert import po2prop, test_convert
 from translate.misc import wStringIO
 from translate.storage import po
@@ -62,6 +64,25 @@ class TestPO2Prop(object):
         print(propfile)
         assert propfile == propexpected
 
+    def test_no_value(self):
+        """check that we can handle keys without value"""
+        posource = '''#: KEY\nmsgctxt "KEY"\nmsgid ""\nmsgstr ""\n'''
+        proptemplate = '''KEY = \n'''
+        propexpected = '''KEY = \n'''
+        propfile = self.merge2prop(proptemplate, posource)
+        print(propfile)
+        assert propfile == propexpected
+
+    @mark.xfail(reason="This doesn't work as expected right now")
+    def test_no_separator(self):
+        """check that we can handle keys without separator"""
+        posource = '''#: KEY\nmsgctxt "KEY"\nmsgid ""\nmsgstr ""\n'''
+        proptemplate = '''KEY\n'''
+        propexpected = '''KEY\n'''
+        propfile = self.merge2prop(proptemplate, posource)
+        print(propfile)
+        assert propfile == propexpected
+
     def test_merging_blank_entries(self):
         """check that we can correctly merge entries that are blank in the template"""
         posource = r'''#: accesskey-accept
@@ -363,6 +384,84 @@ message-multiedit-header[other]={{ n }} gekies
         propfile = self.merge2prop(proptemplate, posource, personality="gaia")
         assert propfile == propexpected
 
+    def test_duplicates(self):
+        """Test back conversion of properties with duplicate units."""
+        # Test entries with same key and value.
+        proptemplate = '''
+key=value
+key=value
+'''
+        posource = r'''#: key
+msgid "value"
+msgstr "Waarde"
+'''
+        propexpected = '''
+key=Waarde
+key=Waarde
+'''
+        propfile = self.merge2prop(proptemplate, posource, personality="mozilla")
+        assert propfile == propexpected
+
+        # Test entries with same key and different value, and single
+        # corresponding entry in PO.
+        proptemplate = '''
+key=value
+key=another value
+'''
+        posource = r'''#: key
+msgid "value"
+msgstr "Waarde"
+'''
+        propexpected = '''
+key=Waarde
+key=Waarde
+'''
+        propfile = self.merge2prop(proptemplate, posource, personality="mozilla")
+        assert propfile == propexpected
+
+        # Test entries with same key and different value, and two different
+        # corresponding entries in PO.
+        proptemplate = '''
+key=value
+key=another value
+'''
+        posource = r'''#: key
+msgid "value"
+msgstr "Valor"
+
+#: key
+msgid "another value"
+msgstr "Outro valor"
+'''
+        propexpected = '''
+key=Valor
+key=Valor
+'''
+        propfile = self.merge2prop(proptemplate, posource, personality="mozilla")
+        assert propfile == propexpected
+
+        # Test entries with same key and different value.
+        proptemplate = '''
+key1=value
+key2=value
+'''
+        posource = r'''#: key1
+msgctxt "key1"
+msgid "value"
+msgstr "Waarde"
+
+#: key2
+msgctxt "key2"
+msgid "value"
+msgstr "Waarde"
+'''
+        propexpected = '''
+key1=Waarde
+key2=Waarde
+'''
+        propfile = self.merge2prop(proptemplate, posource, personality="mozilla")
+        assert propfile == propexpected
+
 
 class TestPO2PropCommand(test_convert.TestConvertCommand, TestPO2Prop):
     """Tests running actual po2prop commands on files"""
