@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 from pytest import importorskip
@@ -12,7 +11,7 @@ from translate.storage import po
 importorskip("aeidon")
 
 
-class TestPO2Sub:
+class TestPO2Sub(object):
 
     def po2sub(self, posource):
         """helper that converts po source to subtitle source without requiring
@@ -21,7 +20,7 @@ class TestPO2Sub:
         inputpo = po.pofile(inputfile)
         convertor = po2sub.resub()
         outputsub = convertor.convertstore(inputpo)
-        return outputsub
+        return outputsub.decode('utf-8')
 
     def merge2sub(self, subsource, posource):
         """helper that merges po translations to subtitle source without
@@ -32,7 +31,7 @@ class TestPO2Sub:
         convertor = po2sub.resub(templatefile, inputpo)
         outputsub = convertor.convertstore()
         print(outputsub)
-        return outputsub
+        return outputsub.decode('utf-8')
 
     def test_subrip(self):
         """test SubRip or .srt files."""
