@@ -1,18 +1,18 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 # po2tiki unit tests
 # Author: Wil Clouser <wclouser@mozilla.com>
 # Date: 2008-12-01
 
+from io import BytesIO
+
 from translate.convert import po2tiki, test_convert
-from translate.misc import wStringIO
 
 
-class TestPo2Tiki:
+class TestPo2Tiki(object):
 
     def test_convertpo(self):
-        inputfile = """
+        inputfile = b"""
 #: translated
 msgid "zero_source"
 msgstr "zero_target"
@@ -21,10 +21,10 @@ msgstr "zero_target"
 msgid "one_source"
 msgstr "one_target"
         """
-        outputfile = wStringIO.StringIO()
+        outputfile = BytesIO()
         po2tiki.convertpo(inputfile, outputfile)
 
-        output = outputfile.getvalue()
+        output = outputfile.getvalue().decode('utf-8')
 
         assert '"one_source" => "one_target",' in output
         assert '"zero_source" => "zero_target",' in output
