@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004-2006 Zuza Software Foundation
@@ -27,7 +26,7 @@ for examples and usage instructions.
 from translate.storage import po, txt
 
 
-class txt2po:
+class txt2po(object):
 
     def __init__(self, duplicatestyle="msgctxt"):
         self.duplicatestyle = duplicatestyle
@@ -49,21 +48,19 @@ class txt2po:
 def converttxt(inputfile, outputfile, templates, duplicatestyle="msgctxt",
                encoding="utf-8", flavour=None):
     """reads in stdin using fromfileclass, converts using convertorclass,
-    writes to stdout"""
+    writes to stdout
+    """
     inputstore = txt.TxtFile(inputfile, encoding=encoding, flavour=flavour)
     convertor = txt2po(duplicatestyle=duplicatestyle)
     outputstore = convertor.convertstore(inputstore)
     if outputstore.isempty():
         return 0
-    outputfile.write(str(outputstore))
+    outputstore.serialize(outputfile)
     return 1
 
 
 def main(argv=None):
     from translate.convert import convert
-    from translate.misc import stdiotell
-    import sys
-    sys.stdout = stdiotell.StdIOWrapper(sys.stdout)
     formats = {"txt": ("po", converttxt), "*": ("po", converttxt)}
     parser = convert.ConvertOptionParser(formats, usepots=True,
                                          description=__doc__)
