@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2007 Zuza Software Foundation
@@ -23,10 +22,14 @@
 .. seealso:: http://en.wikipedia.org/wiki/Hebrew_language
 """
 
+from __future__ import unicode_literals
+
 from translate.lang import common
 
 
 class he(common.Common):
     """This class represents Hebrew."""
 
-    ignoretests = ["startcaps", "simplecaps", "acronyms"]
+    ignoretests = {
+        'all': ["acronyms", "simplecaps", "startcaps"],
+    }
