@@ -1,7 +1,6 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright 2010 Zuza Software Foundation
+# Copyright 2010, 2016 Zuza Software Foundation
 #
 # This file is part of translate.
 #
@@ -23,10 +22,14 @@
 .. seealso:: http://en.wikipedia.org/wiki/Marathi_language
 """
 
+from __future__ import unicode_literals
+
 from translate.lang import common
 
 
 class mr(common.Common):
     """This class represents Marathi."""
 
-    ignoretests = ["startcaps", "simplecaps"]
+    ignoretests = {
+        'all': ["simplecaps", "startcaps"],
+    }
