@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2013 Zuza Software Foundation
@@ -23,10 +22,14 @@
 .. seealso:: http://en.wikipedia.org/wiki/Chinese_language
 """
 
+from __future__ import unicode_literals
+
 from translate.lang.zh import zh
 
 
 class zh_tw(zh):
-    specialchars = u"←→↔×÷©…—‘’“”「」『』【】《》"
+    specialchars = "←→↔×÷©…—‘’“”「」『』【】《》"
 
-    ignoretests = ["startcaps", "simplecaps", "acronyms"]
+    ignoretests = {
+        'all': ["acronyms", "simplecaps", "startcaps"],
+    }
