@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2002-2006 Zuza Software Foundation
@@ -23,14 +22,47 @@ import logging
 import optparse
 import os.path
 import re
+import six
 import sys
 import traceback
-from cStringIO import StringIO
+from collections import OrderedDict
+from io import BytesIO
 
 from translate import __version__
 from translate.misc import progressbar
 
 
+class ProgressBar(object):
+    progress_types = OrderedDict([
+        ("dots", progressbar.DotsProgressBar),
+        ("none", progressbar.NoProgressBar),
+        ("bar", progressbar.HashProgressBar),
+        ("names", progressbar.MessageProgressBar),
+        ("verbose", progressbar.VerboseProgressBar),
+    ])
+
+    def __init__(self, progress_type, allfiles):
+        """Set up a progress bar appropriate to the progress_type and files."""
+        if progress_type in ('bar', 'verbose'):
+            file_count = len(allfiles)
+            self._progressbar = self.progress_types[progress_type](0, file_count)
+            logger = logging.getLogger(os.path.basename(sys.argv[0])).getChild("progress")
+            logger.setLevel(logging.INFO)
+            logger.propagate = False
+            handler = logging.StreamHandler()
+            handler.setLevel(logging.INFO)
+            handler.setFormatter(logging.Formatter())
+            logger.addHandler(handler)
+            logger.info("processing %d files...", file_count)
+        else:
+            self._progressbar = self.progress_types[progress_type]()
+
+    def report_progress(self, filename, success):
+        """Show that we are progressing..."""
+        self._progressbar.amount += 1
+        self._progressbar.show(filename)
+
+
 class ManPageOption(optparse.Option, object):
     ACTIONS = optparse.Option.ACTIONS + ("manpage",)
 
@@ -45,11 +77,8 @@ class ManPageOption(optparse.Option, object):
 
 class ManHelpFormatter(optparse.HelpFormatter):
 
-    def __init__(self,
-            indent_increment=0,
-            max_help_position=0,
-            width=80,
-            short_first=1):
+    def __init__(self, indent_increment=0, max_help_position=0, width=80,
+                 short_first=1):
         optparse.HelpFormatter.__init__(
             self, indent_increment, max_help_position, width, short_first)
 
@@ -72,6 +101,22 @@ class ManHelpFormatter(optparse.HelpFormatter):
         return '\\fB%s\\fP' % ("\\fR, \\fP".join(opts))
 
 
+class StdoutWrapper(object):
+    out = sys.stdout
+
+    def __getattr__(self, name):
+        return getattr(self.out, name)
+
+    def write(self, content):
+        if six.PY3 and isinstance(content, bytes):
+            try:
+                self.out.write(content.decode('utf-8'))
+            except UnicodeDecodeError:
+                self.out.write("Unable to write binary content to the terminal")
+        else:
+            self.out.write(content)
+
+
 class RecursiveOptionParser(optparse.OptionParser, object):
     """A specialized Option Parser for recursing through directories."""
 
@@ -99,9 +144,11 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
     def setmanpageoption(self):
         """creates a manpage option that allows the optionparser to generate a
-        manpage"""
-        manpageoption = ManPageOption(None, "--manpage", dest="manpage",
-                                      default=False, action="manpage",
+        manpage
+        """
+        manpageoption = ManPageOption(
+            None, "--manpage", dest="manpage",
+            default=False, action="manpage",
             help="output a manpage based on the help")
         self.define_option(manpageoption)
 
@@ -112,9 +159,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         formatprog = lambda x: x.replace("%prog", prog)
         formatToolkit = lambda x: x.replace("%prog", "Translate Toolkit")
         result.append('.\\" Autogenerated manpage\n')
-        result.append('.TH %s 1 "%s" "" "%s"\n' % (prog,
-            formatToolkit(self.version),
-            formatToolkit(self.version)))
+        result.append('.TH %s 1 "%s" "" "%s"\n' % (
+            prog, formatToolkit(self.version), formatToolkit(self.version)))
         result.append('.SH NAME\n')
         result.append('%s \\- %s\n' % (self.get_prog_name(),
                                        self.description.split('\n\n')[0]))
@@ -128,7 +174,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         if description_lines:
             result.append('.SH DESCRIPTION\n')
             result.append('\n\n'.join([re.sub('\.\. note::', 'Note:', l)
-                                              for l in description_lines]))
+                                       for l in description_lines]))
         result.append('.SH OPTIONS\n')
         ManHelpFormatter().store_option_strings(self)
         result.append('.PP\n')
@@ -146,7 +192,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
     def set_usage(self, usage=None):
         """sets the usage string - if usage not given, uses getusagestring for
-        each option"""
+        each option
+        """
         if usage is None:
             self.usage = "%prog " + " ".join([self.getusagestring(option) for option in self.option_list])
         else:
@@ -193,8 +240,9 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             return "\\fR[\\fP%s\\fR]\\fP" % optionstring
 
     def define_option(self, option):
-        """Defines the given option, replacing an existing one of the same short
-        name if neccessary..."""
+        """Defines the given option, replacing an existing one of the same
+        short name if neccessary...
+        """
         for short_opt in option._short_opts:
             if self.has_option(short_opt):
                 self.remove_option(short_opt)
@@ -220,13 +268,13 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                         outputformat (string) and processor method.
         """
 
-        inputformats = []
+        self.inputformats = []
         outputformats = []
         templateformats = []
         self.outputoptions = {}
         self.usetemplates = usetemplates
-        for formatgroup, outputoptions in formats.iteritems():
-            if isinstance(formatgroup, (str, unicode)) or formatgroup is None:
+        for formatgroup, outputoptions in six.iteritems(formats):
+            if isinstance(formatgroup, six.string_types) or formatgroup is None:
                 formatgroup = (formatgroup, )
             if not isinstance(formatgroup, tuple):
                 raise ValueError("formatgroups must be tuples or None/str/unicode")
@@ -238,71 +286,66 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             if not isinstance(outputoptions, tuple) or len(outputoptions) != 2:
                 raise ValueError("output options must be tuples of length 2")
             outputformat, processor = outputoptions
-            if not inputformat in inputformats:
-                inputformats.append(inputformat)
-            if not outputformat in outputformats:
+            if inputformat not in self.inputformats:
+                self.inputformats.append(inputformat)
+            if outputformat not in outputformats:
                 outputformats.append(outputformat)
-            if not templateformat in templateformats:
+            if templateformat not in templateformats:
                 templateformats.append(templateformat)
             self.outputoptions[(inputformat, templateformat)] = (outputformat, processor)
-        self.inputformats = inputformats
-        inputformathelp = self.getformathelp(inputformats)
-        inputoption = optparse.Option("-i", "--input", dest="input",
-                default=None, metavar="INPUT",
-                help="read from INPUT in %s" % (inputformathelp))
+        inputformathelp = self.getformathelp(self.inputformats)
+        inputoption = optparse.Option(
+            "-i", "--input", dest="input",
+            default=None, metavar="INPUT",
+            help="read from INPUT in %s" % (inputformathelp))
         inputoption.optionalswitch = True
         inputoption.required = True
         self.define_option(inputoption)
-        excludeoption = optparse.Option("-x", "--exclude", dest="exclude",
-                action="append", type="string", metavar="EXCLUDE",
-                default=["CVS", ".svn", "_darcs", ".git", ".hg", ".bzr"],
-                help="exclude names matching EXCLUDE from input paths")
+        excludeoption = optparse.Option(
+            "-x", "--exclude", dest="exclude",
+            action="append", type="string", metavar="EXCLUDE",
+            default=["CVS", ".svn", "_darcs", ".git", ".hg", ".bzr"],
+            help="exclude names matching EXCLUDE from input paths")
         self.define_option(excludeoption)
         outputformathelp = self.getformathelp(outputformats)
-        outputoption = optparse.Option("-o", "--output", dest="output",
-                default=None, metavar="OUTPUT",
-                help="write to OUTPUT in %s" % (outputformathelp))
+        outputoption = optparse.Option(
+            "-o", "--output", dest="output",
+            default=None, metavar="OUTPUT",
+            help="write to OUTPUT in %s" % (outputformathelp))
         outputoption.optionalswitch = True
         outputoption.required = True
         self.define_option(outputoption)
         if self.usetemplates:
             self.templateformats = templateformats
             templateformathelp = self.getformathelp(self.templateformats)
-            templateoption = optparse.Option("-t", "--template",
+            templateoption = optparse.Option(
+                "-t", "--template",
                 dest="template", default=None, metavar="TEMPLATE",
                 help="read from TEMPLATE in %s" % (templateformathelp))
             self.define_option(templateoption)
 
     def setprogressoptions(self):
         """Sets the progress options."""
-        self.progresstypes = {
-                "none": progressbar.NoProgressBar,
-                "bar": progressbar.HashProgressBar,
-                "dots": progressbar.DotsProgressBar,
-                "names": progressbar.MessageProgressBar,
-                "verbose": progressbar.VerboseProgressBar,
-        }
-        progressoption = optparse.Option(None, "--progress", dest="progress",
-                default="bar",
-                choices=self.progresstypes.keys(), metavar="PROGRESS",
-                help="show progress as: %s" % (", ".join(self.progresstypes)))
+        progressoption = optparse.Option(
+            None, "--progress", dest="progress", default="bar",
+            choices=list(ProgressBar.progress_types.keys()), metavar="PROGRESS",
+            help="show progress as: %s" % (", ".join(ProgressBar.progress_types)))
         self.define_option(progressoption)
 
     def seterrorleveloptions(self):
         """Sets the errorlevel options."""
         self.errorleveltypes = ["none", "message", "exception", "traceback"]
-        errorleveloption = optparse.Option(None, "--errorlevel",
-                dest="errorlevel", default="message",
-                choices=self.errorleveltypes, metavar="ERRORLEVEL",
-                help="show errorlevel as: %s" %
-                     (", ".join(self.errorleveltypes)))
+        errorleveloption = optparse.Option(
+            None, "--errorlevel",
+            dest="errorlevel", default="message",
+            choices=self.errorleveltypes, metavar="ERRORLEVEL",
+            help="show errorlevel as: %s" % (
+                ", ".join(self.errorleveltypes)))
         self.define_option(errorleveloption)
 
     def getformathelp(self, formats):
         """Make a nice help string for describing formats..."""
-        formats = sorted(formats)
-        if None in formats:
-            formats = filter(lambda format: format is not None, formats)
+        formats = sorted([f for f in formats if f is not None])
         if len(formats) == 0:
             return ""
         elif len(formats) == 1:
@@ -321,7 +364,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
     def parse_args(self, args=None, values=None):
         """Parses the command line options, handling implicit input/output
-        args."""
+        args.
+        """
         (options, args) = super(RecursiveOptionParser, self).parse_args(args, values)
         # some intelligent as to what reasonable people might give on the
         # command line
@@ -363,22 +407,22 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             templatebase, templateext = self.splittemplateext(templatepath)
         else:
             templateext = None
-        if (inputext, templateext) in options.outputoptions:
-            return options.outputoptions[inputext, templateext]
-        elif (inputext, "*") in options.outputoptions:
-            outputformat, fileprocessor = options.outputoptions[inputext, "*"]
-        elif ("*", templateext) in options.outputoptions:
-            outputformat, fileprocessor = options.outputoptions["*", templateext]
-        elif ("*", "*") in options.outputoptions:
-            outputformat, fileprocessor = options.outputoptions["*", "*"]
-        elif (inputext, None) in options.outputoptions:
-            return options.outputoptions[inputext, None]
-        elif (None, templateext) in options.outputoptions:
-            return options.outputoptions[None, templateext]
-        elif ("*", None) in options.outputoptions:
-            outputformat, fileprocessor = options.outputoptions["*", None]
-        elif (None, "*") in options.outputoptions:
-            outputformat, fileprocessor = options.outputoptions[None, "*"]
+        if (inputext, templateext) in self.outputoptions:
+            return self.outputoptions[inputext, templateext]
+        elif (inputext, "*") in self.outputoptions:
+            outputformat, fileprocessor = self.outputoptions[inputext, "*"]
+        elif ("*", templateext) in self.outputoptions:
+            outputformat, fileprocessor = self.outputoptions["*", templateext]
+        elif ("*", "*") in self.outputoptions:
+            outputformat, fileprocessor = self.outputoptions["*", "*"]
+        elif (inputext, None) in self.outputoptions:
+            return self.outputoptions[inputext, None]
+        elif (None, templateext) in self.outputoptions:
+            return self.outputoptions[None, templateext]
+        elif ("*", None) in self.outputoptions:
+            outputformat, fileprocessor = self.outputoptions["*", None]
+        elif (None, "*") in self.outputoptions:
+            outputformat, fileprocessor = self.outputoptions[None, "*"]
         else:
             if self.usetemplates:
                 if inputext is None:
@@ -397,7 +441,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 outputformat = inputext
             elif templateext:
                 outputformat = templateext
-            elif ("*", "*") in options.outputoptions:
+            elif ("*", "*") in self.outputoptions:
                 outputformat = None
             else:
                 if self.usetemplates:
@@ -406,23 +450,6 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                     raise ValueError("don't know what to do with input format (no file extension)")
         return outputformat, fileprocessor
 
-    def initprogressbar(self, allfiles, options):
-        """Sets up a progress bar appropriate to the options and files."""
-        if options.progress in ('bar', 'verbose'):
-            self.progressbar = \
-                self.progresstypes[options.progress](0, len(allfiles))
-            # should use .getChild("progress") but that is only in 2.7
-            logger = logging.getLogger(self.get_prog_name() + ".progress")
-            logger.setLevel(logging.INFO)
-            logger.propagate = False
-            handler = logging.StreamHandler()
-            handler.setLevel(logging.INFO)
-            handler.setFormatter(logging.Formatter())
-            logger.addHandler(handler)
-            logger.info("processing %d files...", len(allfiles))
-        else:
-            self.progressbar = self.progresstypes[options.progress]()
-
     def getfullinputpath(self, options, inputpath):
         """Gets the absolute path to an input file."""
         if options.input:
@@ -449,12 +476,9 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
     def run(self):
         """Parses the arguments, and runs recursiveprocess with the resulting
-        options..."""
+        options...
+        """
         (options, args) = self.parse_args()
-        # this is so derived classes can modify the inputformats etc based on
-        # the options
-        options.inputformats = self.inputformats
-        options.outputoptions = self.outputoptions
         self.recursiveprocess(options)
 
     def recursiveprocess(self, options):
@@ -483,7 +507,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         options.recursivetemplate = (self.usetemplates and
                                      self.isrecursive(options.template, 'template') and
                                      getattr(options, "allowrecursivetemplate", True))
-        self.initprogressbar(inputfiles, options)
+        progress_bar = ProgressBar(options.progress, inputfiles)
         for inputpath in inputfiles:
             try:
                 templatepath = self.gettemplatename(options, inputpath)
@@ -502,9 +526,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 fulloutputpath = self.getfulloutputpath(options, outputpath)
                 if options.recursiveoutput and outputpath:
                     self.checkoutputsubdir(options, os.path.dirname(outputpath))
-            except Exception as error:
-                if isinstance(error, KeyboardInterrupt):
-                    raise
+            except Exception:
                 self.warning("Couldn't handle input file %s" %
                              inputpath, options, sys.exc_info())
                 continue
@@ -512,35 +534,33 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 success = self.processfile(fileprocessor, options,
                                            fullinputpath, fulloutputpath,
                                            fulltemplatepath)
-            except Exception as error:
-                if isinstance(error, KeyboardInterrupt):
-                    raise
+            except Exception:
                 self.warning("Error processing: input %s, output %s, template %s" %
                              (fullinputpath, fulloutputpath,
                               fulltemplatepath), options, sys.exc_info())
                 success = False
-            self.reportprogress(inputpath, success)
-        del self.progressbar
+            progress_bar.report_progress(inputpath, success)
+        del progress_bar
 
     def openinputfile(self, options, fullinputpath):
         """Opens the input file."""
         if fullinputpath is None:
             return sys.stdin
-        return open(fullinputpath, 'r')
+        return open(fullinputpath, 'rb')
 
     def openoutputfile(self, options, fulloutputpath):
         """Opens the output file."""
         if fulloutputpath is None:
-            return sys.stdout
-        return open(fulloutputpath, 'w')
+            return StdoutWrapper()
+        return open(fulloutputpath, 'wb')
 
     def opentempoutputfile(self, options, fulloutputpath):
         """Opens a temporary output file."""
-        return StringIO()
+        return BytesIO()
 
     def finalizetempoutputfile(self, options, outputfile, fulloutputpath):
         """Write the temp outputfile to its final destination."""
-        outputfile.reset()
+        outputfile.seek(0, 0)
         outputstring = outputfile.read()
         outputfile = self.openoutputfile(options, fulloutputpath)
         outputfile.write(outputstring)
@@ -550,7 +570,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
         """Opens the template file (if required)."""
         if fulltemplatepath is not None:
             if os.path.isfile(fulltemplatepath):
-                return open(fulltemplatepath, 'r')
+                return open(fulltemplatepath, 'rb')
             else:
                 self.warning("missing template file %s" % fulltemplatepath)
         return None
@@ -582,11 +602,6 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 os.unlink(fulloutputpath)
             return False
 
-    def reportprogress(self, filename, success):
-        """Shows that we are progressing..."""
-        self.progressbar.amount += 1
-        self.progressbar.show(filename)
-
     def mkdir(self, parent, subdir):
         """Makes a subdirectory (recursively if neccessary)."""
         if not os.path.isdir(parent):
@@ -601,7 +616,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
 
     def checkoutputsubdir(self, options, subdir):
         """Checks to see if subdir under options.output needs to be created,
-        creates if neccessary."""
+        creates if neccessary.
+        """
         fullpath = os.path.join(options.output, subdir)
         if not os.path.isdir(fullpath):
             self.mkdir(options.output, subdir)
@@ -648,7 +664,7 @@ class RecursiveOptionParser(optparse.OptionParser, object):
                 if os.path.isdir(fullinputpath):
                     dirs.append(inputpath)
                 elif os.path.isfile(fullinputpath):
-                    if not self.isvalidinputname(options, name):
+                    if not self.isvalidinputname(name):
                         # only handle names that match recognized input
                         # file extensions
                         continue
@@ -691,14 +707,14 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             return options.template
         inputbase, inputext = self.splitinputext(inputname)
         if options.template:
-            for inputext1, templateext1 in options.outputoptions:
+            for inputext1, templateext1 in self.outputoptions:
                 if inputext == inputext1:
                     if templateext1:
                         templatepath = inputbase + os.extsep + templateext1
                         if self.templateexists(options, templatepath):
                             return templatepath
-            if "*" in options.inputformats:
-                for inputext1, templateext1 in options.outputoptions:
+            if "*" in self.inputformats:
+                for inputext1, templateext1 in self.outputoptions:
                     if (inputext == inputext1) or (inputext1 == "*"):
                         if templateext1 == "*":
                             templatepath = inputname
@@ -720,8 +736,8 @@ class RecursiveOptionParser(optparse.OptionParser, object):
             outputname += os.extsep + outputformat
         return outputname
 
-    def isvalidinputname(self, options, inputname):
+    def isvalidinputname(self, inputname):
         """Checks if this is a valid input filename."""
         inputbase, inputext = self.splitinputext(inputname)
-        return ((inputext in options.inputformats) or
-                ("*" in options.inputformats))
+        return ((inputext in self.inputformats) or
+                ("*" in self.inputformats))
