@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2004, 2005, 2010 Zuza Software Foundation
@@ -19,12 +18,14 @@
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 """Progress bar utilities for reporting feedback on the progress of an
-application."""
+application.
+"""
 
 
 class DotsProgressBar:
     """An ultra-simple progress indicator that just writes a dot for each
-    action"""
+    action
+    """
 
     def __init__(self):
         import sys
@@ -102,7 +103,8 @@ class ProgressBar:
 
 class MessageProgressBar(ProgressBar):
     """A ProgressBar that just writes out the messages without any progress
-    display"""
+    display
+    """
 
     def __init__(self, *args, **kwargs):
         import sys
@@ -115,8 +117,8 @@ class MessageProgressBar(ProgressBar):
 
 
 class HashProgressBar(ProgressBar):
-    """A ProgressBar which knows how to go back to the beginning of the
-    line."""
+    """A ProgressBar which knows how to go back to the beginning of the line.
+    """
 
     def __init__(self, *args, **kwargs):
         import sys
@@ -157,6 +159,7 @@ def test(progressbar):
         progressbar.show("Some message")
         time.sleep(0.2)
 
+
 if __name__ == '__main__':
     p = HashProgressBar(0, 100, 50)
     test(p)
