@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2002-2006 Zuza Software Foundation
@@ -18,17 +17,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
-"""String processing utilities for extracting strings with various kinds
-of delimiters"""
+"""String processing utilities for extracting strings with various kinds of
+delimiters
+"""
 
 import logging
+import six
 
 from six.moves import html_entities
 
 
 def find_all(searchin, substr):
-    """Returns a list of locations where substr occurs in searchin
-    locations are not allowed to overlap"""
+    """Returns a list of locations where substr occurs in searchin locations
+    are not allowed to overlap
+    """
     location = 0
     locations = []
     substr_len = len(substr)
@@ -152,7 +154,7 @@ def extractwithoutquotes(source, startdelim, enddelim, escape=None,
                         replace_escape = includeescapes(section[epos:epos + lenescape + 1])
                         # TODO: deprecate old method of returning boolean from
                         # includeescape, by removing this if block
-                        if not isinstance(replace_escape, basestring):
+                        if not isinstance(replace_escape, six.string_types):
                             if replace_escape:
                                 replace_escape = section[epos:epos + lenescape + 1]
                             else:
@@ -196,6 +198,7 @@ def _encode_entity_char(char, codepoint2name):
     else:
         return char
 
+
 def entityencode(source, codepoint2name):
     """Encode ``source`` using entities from ``codepoint2name``.
 
@@ -216,8 +219,8 @@ def entityencode(source, codepoint2name):
                 output += "&" + possibleentity + ";"
                 inentity = False
             elif char == " ":
-                output += _encode_entity_char("&", codepoint2name) + \
-                          entityencode(possibleentity + char, codepoint2name)
+                output += (_encode_entity_char("&", codepoint2name) +
+                           entityencode(possibleentity + char, codepoint2name))
                 inentity = False
             else:
                 possibleentity += char
@@ -225,8 +228,8 @@ def entityencode(source, codepoint2name):
             output += _encode_entity_char(char, codepoint2name)
     if inentity:
         # Handle nonentities at end of string.
-        output += _encode_entity_char("&", codepoint2name) + \
-                  entityencode(possibleentity, codepoint2name)
+        output += (_encode_entity_char("&", codepoint2name) +
+                   entityencode(possibleentity, codepoint2name))
 
     return output
 
@@ -239,6 +242,7 @@ def _has_entity_end(source):
             return False
     return False
 
+
 def entitydecode(source, name2codepoint):
     """Decode ``source`` using entities from ``name2codepoint``.
 
@@ -259,7 +263,7 @@ def entitydecode(source, name2codepoint):
             if char == ";":
                 if (len(possibleentity) > 0 and
                     possibleentity in name2codepoint):
-                    entchar = unichr(name2codepoint[possibleentity])
+                    entchar = six.unichr(name2codepoint[possibleentity])
                     if entchar == u'&' and _has_entity_end(source[i+1:]):
                         output += "&" + possibleentity + ";"
                     else:
@@ -282,7 +286,7 @@ def entitydecode(source, name2codepoint):
 
 
 def htmlentityencode(source):
-    """Encode ``source`` using HTML entities e.g. © -> ``&copy;``
+    u"""Encode ``source`` using HTML entities e.g. © -> ``&copy;``
 
     :param unicode source: Source string to encode
     """
@@ -290,7 +294,7 @@ def htmlentityencode(source):
 
 
 def htmlentitydecode(source):
-    """Decode source using HTML entities e.g. ``&copy;`` -> ©.
+    u"""Decode source using HTML entities e.g. ``&copy;`` -> ©.
 
     :param unicode source: Source string to decode
     """
@@ -315,8 +319,8 @@ def javapropertiesencode(source):
     return output
 
 
-def mozillapropertiesencode(source):
-    """Encodes source in the escaped-unicode encoding used by Mozilla
+def java_utf8_properties_encode(source):
+    """Encodes source in the escaped-unicode encoding used by java utf-8
     .properties files.
     """
     output = u""
@@ -327,12 +331,14 @@ def mozillapropertiesencode(source):
             output += char
     return output
 
+
 def escapespace(char):
     assert(len(char) == 1)
     if char.isspace():
-        return u"\\u%04X" %(ord(char))
+        return u"\\u%04X" % ord(char)
     return char
 
+
 def mozillaescapemarginspaces(source):
     """Escape leading and trailing spaces for Mozilla .properties files."""
     if not source:
@@ -349,6 +355,7 @@ def mozillaescapemarginspaces(source):
     else:
         return escapespace(source[0]) + source[1:-1] + escapespace(source[-1])
 
+
 propertyescapes = {
     # escapes that are self-escaping
     "\\": "\\", "'": "'", '"': '"',
@@ -365,7 +372,7 @@ controlchars = {
 
 def escapecontrols(source):
     """escape control characters in the given string"""
-    for key, value in controlchars.iteritems():
+    for key, value in six.iteritems(controlchars):
         source = source.replace(key, value)
     return source
 
@@ -388,11 +395,11 @@ def propertiesdecode(source):
         otherwise an escaped control character.
         """
         if 32 <= i:
-            return unichr(i)
-        elif unichr(i) in controlchars:
+            return six.unichr(i)
+        elif six.unichr(i) in controlchars:
             # we just return the character, unescaped
             # if people want to escape them they can use escapecontrols
-            return unichr(i)
+            return six.unichr(i)
         return "\\u%04x" % i
 
     while s < len(source):
