@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2006-2009 Zuza Software Foundation
@@ -20,30 +19,29 @@
 
 """Base classes for storage interfaces."""
 
+import codecs
 import logging
+import six
 try:
     import cPickle as pickle
 except ImportError:
     import pickle
+from io import BytesIO
 
 from translate.misc.multistring import multistring
-from translate.storage.placeables import (StringElem, general,
-                                          parse as rich_parse)
+from translate.storage.placeables import StringElem, parse as rich_parse
 from translate.storage.workflow import StateEnum as states
 
-
-def force_override(method, baseclass):
-    """Forces derived classes to override method."""
-
-    if type(method.im_self) == type(baseclass):
-        # then this is a classmethod and im_self is the actual class
-        actualclass = method.im_self
-    else:
-        actualclass = method.im_class
-    if actualclass != baseclass:
-        raise NotImplementedError(
-            "%s does not reimplement %s as required by %s" %
-            (actualclass.__name__, method.__name__, baseclass.__name__))
+# Simple BOM based encoding detection
+ENCODING_BOMS = (
+    (codecs.BOM_UTF8, 'utf-8-sig'),
+    (codecs.BOM_UTF16, 'utf-16'),
+    (codecs.BOM_UTF16_BE, 'utf-16-be'),
+    (codecs.BOM_UTF16_LE, 'utf-16-le'),
+    (codecs.BOM_UTF32, 'utf-32'),
+    (codecs.BOM_UTF32_BE, 'utf-32-be'),
+    (codecs.BOM_UTF32_LE, 'utf-32-le'),
+)
 
 
 class ParseError(Exception):
@@ -55,6 +53,7 @@ class ParseError(Exception):
         return repr(self.inner_exc)
 
 
+@six.python_2_unicode_compatible
 class TranslationUnit(object):
     """Base class for translation units.
 
@@ -140,35 +139,38 @@ class TranslationUnit(object):
         """
         return self.source == other.source and self.target == other.target
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __hash__(self):
+        return None
+
     def __str__(self):
-        """Converts to a string representation that can be parsed back using
-        :meth:`~.TranslationStore.parsestring`."""
-        # no point in pickling store object, so let's hide it for a while.
-        store = getattr(self, "_store", None)
-        self._store = None
-        dump = pickle.dumps(self)
-        self._store = store
-        return dump
+        """Converts to a string representation. Most often overriden by subclasses."""
+        # no point in showing store object.
+        return ", ".join([
+            "%s: %s" % (k, self.__dict__[k]) for k in sorted(self.__dict__.keys()) if k != '_store'
+        ])
 
     @classmethod
     def rich_to_multistring(cls, elem_list):
         """Convert a "rich" string tree to a ``multistring``:
 
-           >>> from translate.storage.placeables.interfaces import X
-           >>> rich = [StringElem(['foo', X(id='xxx', sub=[' ']), 'bar'])]
-           >>> TranslationUnit.rich_to_multistring(rich)
-           multistring(u'foo bar')
+        >>> from translate.storage.placeables.interfaces import X
+        >>> rich = [StringElem(['foo', X(id='xxx', sub=[' ']), 'bar'])]
+        >>> TranslationUnit.rich_to_multistring(rich)
+        multistring(u'foo bar')
         """
-        return multistring([unicode(elem) for elem in elem_list])
+        return multistring([six.text_type(elem) for elem in elem_list])
 
     def multistring_to_rich(self, mulstring):
         """Convert a multistring to a list of "rich" string trees:
 
-           >>> target = multistring([u'foo', u'bar', u'baz'])
-           >>> TranslationUnit.multistring_to_rich(target)
-           [<StringElem([<StringElem([u'foo'])>])>,
-            <StringElem([<StringElem([u'bar'])>])>,
-            <StringElem([<StringElem([u'baz'])>])>]
+        >>> target = multistring([u'foo', u'bar', u'baz'])
+        >>> TranslationUnit.multistring_to_rich(target)
+        [<StringElem([<StringElem([u'foo'])>])>,
+         <StringElem([<StringElem([u'bar'])>])>,
+         <StringElem([<StringElem([u'baz'])>])>]
         """
         if isinstance(mulstring, multistring):
             return [rich_parse(s, self.rich_parsers) for s in mulstring.strings]
@@ -255,8 +257,9 @@ class TranslationUnit(object):
     def setid(self, value):
         """Sets the unique identified for this unit.
 
-        only implemented if format allows ids independant from other
-        unit properties like source or context"""
+        only implemented if format allows ids independant from other unit
+        properties like source or context
+        """
         pass
 
     def getlocations(self):
@@ -440,7 +443,8 @@ class TranslationUnit(object):
     @classmethod
     def buildfromunit(cls, unit):
         """Build a native unit from a foreign unit, preserving as much
-        information as possible."""
+        information as possible.
+        """
         if type(unit) == cls and hasattr(unit, "copy") and callable(unit.copy):
             return unit.copy()
         newunit = cls(unit.source)
@@ -460,7 +464,7 @@ class TranslationUnit(object):
     def get_state_id(self, n=None):
         if n is None:
             n = self.get_state_n()
-        for state_id, state_range in self.STATE.iteritems():
+        for state_id, state_range in six.iteritems(self.STATE):
             if state_range[0] <= n < state_range[1]:
                 return state_id
         if self.STATE:
@@ -479,13 +483,14 @@ class TranslationUnit(object):
 
     def infer_state(self):
         """Empty method that should be overridden in sub-classes to infer the
-            current state(_n) of the unit from its current state."""
+        current state(_n) of the unit from its current state.
+        """
         pass
 
 
 class TranslationStore(object):
-    """Base class for stores for multiple translation units of type
-    UnitClass."""
+    """Base class for stores for multiple translation units of type UnitClass.
+    """
 
     UnitClass = TranslationUnit
     """The class of units that will be instantiated and used by this class"""
@@ -501,14 +506,31 @@ class TranslationStore(object):
     """Indicates if format can store suggestions and alternative translation
     for a unit"""
 
+    default_encoding = 'utf-8'
     sourcelanguage = None
     targetlanguage = None
 
-    def __init__(self, unitclass=None):
+    def __init__(self, unitclass=None, encoding=None):
         """Construct a blank TranslationStore."""
         self.units = []
         if unitclass:
             self.UnitClass = unitclass
+        self._encoding = encoding
+
+    @property
+    def encoding(self):
+        # self._encoding is either defined by __init__ or auto-detected from parsed file
+        if self._encoding == 'auto':
+            return 'utf-8'
+        return self._encoding or self.default_encoding
+
+    @encoding.setter
+    def encoding(self, value):
+        if value == "CHARSET" or value is None:
+            return
+        if value == 'ascii':
+            value = 'utf-8'
+        self._encoding = value
 
     def getsourcelanguage(self):
         """Get the source language for this store."""
@@ -631,9 +653,9 @@ class TranslationStore(object):
             remove_unit(unit.source)
 
         for location in unit.getlocations():
-            if location in self.locationindex \
-                   and self.locationindex[location] is not None \
-                   and self.locationindex[location] == unit:
+            if (location in self.locationindex and
+                self.locationindex[location] is not None and
+                self.locationindex[location] == unit):
                 del(self.locationindex[location])
 
     def add_unit_to_index(self, unit):
@@ -641,7 +663,7 @@ class TranslationStore(object):
         self.id_index[unit.getid()] = unit
 
         def insert_unit(source):
-            if not source in self.sourceindex:
+            if source not in self.sourceindex:
                 self.sourceindex[source] = [unit]
             else:
                 self.sourceindex[source].append(unit)
@@ -653,16 +675,16 @@ class TranslationStore(object):
             insert_unit(unit.source)
 
         for location in unit.getlocations():
-            if location in self.locationindex:
-                # if sources aren't unique, don't use them
-                #FIXME: maybe better store a list of units like sourceindex
-                self.locationindex[location] = None
-            else:
+            # If locations aren't unique, keep the first unit.
+            if location not in self.locationindex:
+                #FIXME: maybe better store a list of units like sourceindex in
+                # case there are several units with the same location.
                 self.locationindex[location] = unit
 
     def makeindex(self):
         """Indexes the items in this store. At least .sourceindex should be
-        useful."""
+        useful.
+        """
         self.locationindex = {}
         self.sourceindex = {}
         self.id_index = {}
@@ -683,23 +705,28 @@ class TranslationStore(object):
 
     def __getstate__(self):
         odict = self.__dict__.copy()
+        # fileobj is generally not picklable
         odict['fileobj'] = None
         return odict
 
-    def __setstate__(self, dict):
-        self.__dict__.update(dict)
-        if getattr(self, "filename", False):
-            self.fileobj = open(self.filename)
-
     def __str__(self):
-        """Converts to a string representation that can be parsed back using
-        :meth:`~.TranslationStore.parsestring`."""
-        # We can't pickle fileobj if it is there, so let's hide it for a while.
-        fileobj = getattr(self, "fileobj", None)
-        self.fileobj = None
-        dump = pickle.dumps(self)
-        self.fileobj = fileobj
-        return dump
+        # This allows the old str(store) method for serialization to be kept
+        # for compatibility purpose.
+        if six.PY2:
+            return self.__bytes__()
+        return super(TranslationStore, self).__str__()
+
+    def __bytes__(self):
+        out = BytesIO()
+        self.serialize(out)
+        return out.getvalue()
+
+    def serialize(self, out):
+        """Converts to a bytes representation that can be parsed back using
+        :meth:`~.TranslationStore.parsestring`.
+        `out` should be an open file-like objects to write to.
+        """
+        out.write(pickle.dumps(self))
 
     def isempty(self):
         """Return True if the object doesn't contain any translation units."""
@@ -712,7 +739,8 @@ class TranslationStore(object):
 
     def _assignname(self):
         """Tries to work out what the name of the filesystem file is and
-        assigns it to .filename."""
+        assigns it to .filename.
+        """
         fileobj = getattr(self, "fileobj", None)
         if fileobj:
             filename = getattr(fileobj, "name",
@@ -725,45 +753,76 @@ class TranslationStore(object):
         """Convert the string representation back to an object."""
         newstore = cls()
         if storestring:
+            if isinstance(storestring, six.text_type):
+                # parse() is expecting bytes
+                storestring = storestring.encode(cls.default_encoding)
             newstore.parse(storestring)
         return newstore
 
+    def fallback_detection(self, text):
+        """
+        Simple detection based on BOM in case chardet is not available.
+        """
+        for bom, encoding in ENCODING_BOMS:
+            if text.startswith(bom):
+                return {
+                    'encoding': encoding,
+                    'confidence': 1.0
+                }
+        return None
+
     def detect_encoding(self, text, default_encodings=None):
+        """
+        Try to detect a file encoding from `text`, using either the chardet lib
+        or by trying to decode the file.
+        """
         if not default_encodings:
             default_encodings = ['utf-8']
         try:
             import chardet
-            # many false complaints with ellipse (see bug 1825)
-            detected_encoding = chardet.detect(text.replace("…", ""))
+        except ImportError:
+            detected_encoding = self.fallback_detection(text)
+        else:
+            # many false complaints with ellipse (…) (see bug 1825)
+            detected_encoding = chardet.detect(text.replace(b"\xe2\x80\xa6", b""))
             if detected_encoding['confidence'] < 0.48:
                 detected_encoding = None
             elif detected_encoding['encoding'] == 'ascii':
-                detected_encoding['encoding'] = 'utf-8'
-        except ImportError:
-            detected_encoding = None
+                detected_encoding['encoding'] = self.encoding
+            else:
+                detected_encoding['encoding'] = detected_encoding['encoding'].lower()
 
         encodings = []
-        if self.encoding == 'auto':
+        # Purposefully accessed the internal _encoding, as encoding is never 'auto'
+        if self._encoding == 'auto':
             if detected_encoding and detected_encoding['encoding'] not in encodings:
                 encodings.append(detected_encoding['encoding'])
             for encoding in default_encodings:
                 if encoding not in encodings:
                     encodings.append(encoding)
-        else:
-            encodings.append(self.encoding)
-            if (detected_encoding and
-                    detected_encoding['encoding'] != self.encoding and
-                    detected_encoding['confidence'] != 1.0):
+        elif detected_encoding:
+            if '-' in detected_encoding['encoding']:
+                encoding, suffix = detected_encoding['encoding'].rsplit('-', 1)
+            else:
+                encoding = detected_encoding['encoding']
+                suffix = None
+
+            # Different charset, just with BOM
+            if encoding == self.encoding and suffix == 'sig':
+                encodings.append(detected_encoding['encoding'])
+            elif detected_encoding['encoding'] != self.encoding:
                 logging.warn("trying to parse %s with encoding: %s but "
                              "detected encoding is %s (confidence: %s)",
                              self.filename, self.encoding,
                              detected_encoding['encoding'],
                              detected_encoding['confidence'])
             encodings.append(self.encoding)
+        else:
+            encodings.append(self.encoding)
 
         for encoding in encodings:
             try:
-                r_text = unicode(text, encoding)
+                r_text = six.text_type(text, encoding)
                 r_encoding = encoding
                 break
             except UnicodeDecodeError:
@@ -779,47 +838,37 @@ class TranslationStore(object):
 
     def savefile(self, storefile):
         """Write the string representation to the given file (or filename)."""
-        storestring = str(self)
-        if isinstance(storefile, basestring):
-            mode = 'w'
-            if self._binary:
-                mode = 'wb'
-            storefile = open(storefile, mode)
+        if isinstance(storefile, six.string_types):
+            storefile = open(storefile, 'wb')
         self.fileobj = storefile
         self._assignname()
-        storefile.write(storestring)
+        self.serialize(storefile)
         storefile.close()
 
     def save(self):
-        """Save to the file that data was originally read from, if
-        available."""
+        """Save to the file that data was originally read from, if available.
+        """
         fileobj = getattr(self, "fileobj", None)
-        mode = 'w'
-        if self._binary:
-            mode = 'wb'
         if not fileobj:
-            filename = getattr(self, "filename", None)
-            if filename:
-                fileobj = file(filename, mode)
+            if hasattr(self, "filename"):
+                fileobj = open(self.filename, 'wb')
         else:
             fileobj.close()
             filename = getattr(fileobj, "name",
                                getattr(fileobj, "filename", None))
             if not filename:
                 raise ValueError("No file or filename to save to")
-            fileobj = fileobj.__class__(filename, mode)
+            fileobj = open(filename, 'wb')
         self.savefile(fileobj)
 
     @classmethod
     def parsefile(cls, storefile):
         """Reads the given file (or opens the given filename) and parses back
-        to an object."""
-        mode = 'r'
-        if cls._binary:
-            mode = 'rb'
-        if isinstance(storefile, basestring):
-            storefile = open(storefile, mode)
-        mode = getattr(storefile, "mode", mode)
+        to an object.
+        """
+        if isinstance(storefile, six.string_types):
+            storefile = open(storefile, 'rb')
+        mode = getattr(storefile, "mode", 'rb')
         #For some reason GzipFile returns 1, so we have to test for that here
         if mode == 1 or "r" in mode:
             storestring = storefile.read()
