@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2002-2011 Zuza Software Foundation
@@ -21,7 +20,8 @@
 """Classes for the support of Gettext .po and .pot files.
 
 This implementation assumes that cpo is working. This should not be used
-directly, but can be used once cpo has been established to work."""
+directly, but can be used once cpo has been established to work.
+"""
 
 #TODO:
 # - handle headerless PO files better
@@ -29,13 +29,16 @@ directly, but can be used once cpo has been established to work."""
 # - accept only unicodes everywhere
 
 import copy
+import logging
 import re
-from cStringIO import StringIO
+import six
 
 from translate.lang import data
 from translate.misc.multistring import multistring
-from translate.storage import base, cpo, pocommon, poparser
-from translate.storage.pocommon import encodingToUse
+from translate.storage import base, cpo, pocommon
+
+
+logger = logging.getLogger(__name__)
 
 
 lsep = " "
@@ -48,6 +51,7 @@ msgstr ""
 '''
 
 
+@six.python_2_unicode_compatible
 class pounit(pocommon.pounit):
     # othercomments = []      #   # this is another comment
     # automaticcomments = []  #   #. comment extracted from the source code
@@ -65,9 +69,8 @@ class pounit(pocommon.pounit):
     # fashion
     __shallow__ = ['_store']
 
-    def __init__(self, source=None, encoding="UTF-8"):
+    def __init__(self, source=None, **kwargs):
         pocommon.pounit.__init__(self, source)
-        self._encoding = encodingToUse(encoding)
         self._initallcomments(blankall=True)
         self._msgctxt = u""
 
@@ -92,7 +95,7 @@ class pounit(pocommon.pounit):
         source = source or u""
         if isinstance(source, multistring):
             self._source = source
-        elif isinstance(source, unicode):
+        elif isinstance(source, six.text_type):
             self._source = source
         else:
             #unicode, list, dict
@@ -171,7 +174,7 @@ class pounit(pocommon.pounit):
         # self.__shallow__
         shallow = set(self.__shallow__)
         # Make deep copies of all members which are not in shallow
-        for key, value in self.__dict__.iteritems():
+        for key, value in six.iteritems(self.__dict__):
             if key not in shallow:
                 setattr(new_unit, key, copy.deepcopy(value))
         # Make shallow copies of all members which are in shallow
@@ -206,13 +209,13 @@ class pounit(pocommon.pounit):
         """
 
         def mergelists(list1, list2, split=False):
-            #decode where necessary
-            if unicode in [type(item) for item in list2] + [type(item) for item in list1]:
+            # Decode where necessary (either all bytestrings or all unicode)
+            if six.text_type in [type(item) for item in list2] + [type(item) for item in list1]:
                 for position, item in enumerate(list1):
-                    if isinstance(item, str):
+                    if isinstance(item, bytes):
                         list1[position] = item.decode("utf-8")
                 for position, item in enumerate(list2):
-                    if isinstance(item, str):
+                    if isinstance(item, bytes):
                         list2[position] = item.decode("utf-8")
 
             #Determine the newline style of list2
@@ -232,7 +235,7 @@ class pounit(pocommon.pounit):
                     splitlist1.extend(item.split())
                 for item in list2:
                     splitlist2.extend(item.split())
-                list1.extend([item for item in splitlist2 if not item in splitlist1])
+                list1.extend([item for item in splitlist2 if item not in splitlist1])
             else:
                 #Normal merge, but conform to list1 newline style
                 if list1 != list2:
@@ -288,7 +291,6 @@ class pounit(pocommon.pounit):
 
     def hasmarkedcomment(self, commentmarker):
         """Check whether the given comment marker is present as # (commentmarker) ..."""
-#        raise DeprecationWarning
         commentmarker = "(%s)" % commentmarker
         for comment in self.othercomments:
             if comment.startswith(commentmarker):
@@ -328,10 +330,6 @@ class pounit(pocommon.pounit):
         source = self.source
         return isinstance(source, multistring) and len(source.strings) > 1
 
-    def parse(self, src):
-        raise DeprecationWarning("Should not be parsing with a unit")
-        return poparser.parse_unit(poparser.ParseState(StringIO(src), pounit), self)
-
     def __str__(self):
         """convert to a string. double check that unicode is handled somehow here"""
         _cpo_unit = cpo.pounit.buildfromunit(self)
@@ -392,7 +390,8 @@ class pounit(pocommon.pounit):
     @classmethod
     def buildfromunit(cls, unit):
         """Build a native unit from a foreign unit, preserving as much
-        information as possible."""
+        information as possible.
+        """
         if type(unit) == cls and hasattr(unit, "copy") and callable(unit.copy):
             return unit.copy()
         elif isinstance(unit, pocommon.pounit):
@@ -426,66 +425,32 @@ class pounit(pocommon.pounit):
 
 class pofile(pocommon.pofile):
     """A .po file containing various units"""
+
     UnitClass = pounit
 
-    def changeencoding(self, newencoding):
-        """Deprecated: changes the encoding on the file."""
-        # This should not be here but in poheader. It also shouldn't mangle the
-        # header itself, but use poheader methods. All users are removed, so
-        # we can deprecate after one release.
-        raise DeprecationWarning
-
-        self._encoding = encodingToUse(newencoding)
-        if not self.units:
-            return
-        header = self.header()
-        if not header or header.isblank():
-            return
-        charsetline = None
-        headerstr = header.target
-        for line in headerstr.split("\n"):
-            if not ":" in line:
-                continue
-            key, value = line.strip().split(":", 1)
-            if key.strip() != "Content-Type":
-                continue
-            charsetline = line
-        if charsetline is None:
-            headerstr += "Content-Type: text/plain; charset=%s" % self._encoding
-        else:
-            charset = re.search("charset=([^ ]*)", charsetline)
-            if charset is None:
-                newcharsetline = charsetline
-                if not newcharsetline.strip().endswith(";"):
-                    newcharsetline += ";"
-                newcharsetline += " charset=%s" % self._encoding
-            else:
-                charset = charset.group(1)
-                newcharsetline = charsetline.replace("charset=%s" % charset, "charset=%s" % self._encoding, 1)
-            headerstr = headerstr.replace(charsetline, newcharsetline, 1)
-        header.target = headerstr
-
     def _build_self_from_cpo(self):
         """Builds up this store from the internal cpo store.
 
         A user must ensure that self._cpo_store already exists, and that it is
-        deleted afterwards."""
+        deleted afterwards.
+        """
         for unit in self._cpo_store.units:
             self.addunit(self.UnitClass.buildfromunit(unit))
-        self._encoding = self._cpo_store._encoding
+        self.encoding = self._cpo_store.encoding
 
     def _build_cpo_from_self(self):
         """Builds the internal cpo store from the data in self.
 
         A user must ensure that self._cpo_store does not exist, and should
-        delete it after using it."""
+        delete it after using it.
+        """
         self._cpo_store = cpo.pofile(noheader=True)
         for unit in self.units:
             if not unit.isblank():
-                self._cpo_store.addunit(cpo.pofile.UnitClass.buildfromunit(unit, self._encoding))
+                self._cpo_store.addunit(cpo.pofile.UnitClass.buildfromunit(unit, self.encoding))
         if not self._cpo_store.header():
             #only add a temporary header
-            self._cpo_store.makeheader(charset=self._encoding, encoding="8bit")
+            self._cpo_store.makeheader(charset=self.encoding, encoding="8bit")
 
     def parse(self, input):
         """Parses the given file or file source string."""
@@ -494,16 +459,10 @@ class pofile(pocommon.pofile):
                 self.filename = input.name
             elif not getattr(self, 'filename', ''):
                 self.filename = ''
-            tmp_header_added = False
-#            if isinstance(input, str) and '"Content-Type: text/plain; charset=' not in input[:200]:
-#                input = basic_header + input
-#                tmp_header_added = True
             self.units = []
             self._cpo_store = cpo.pofile(input, noheader=True)
             self._build_self_from_cpo()
             del self._cpo_store
-            if tmp_header_added:
-                self.units = self.units[1:]
         except Exception as e:
             raise base.ParseError(e)
 
@@ -539,7 +498,13 @@ class pofile(pocommon.pofile):
                         origpo._msgctxt += " ".join(origpo.getlocations())
                         markedpos.append(thepo)
                     thepo._msgctxt += " ".join(thepo.getlocations())
-                    uniqueunits.append(thepo)
+                    if not thepo._msgctxt == id_dict[id]._msgctxt:
+                        uniqueunits.append(thepo)
+                    else:
+                        logger.warn(
+                            "Duplicate unit found with msgctx of '%s' and source '%s'",
+                            thepo._msgctxt,
+                            thepo.source)
             else:
                 if not id:
                     if duplicatestyle == "merge":
@@ -550,15 +515,14 @@ class pofile(pocommon.pofile):
                 uniqueunits.append(thepo)
         self.units = uniqueunits
 
-    def __str__(self):
-        """Convert to a string. double check that unicode is handled somehow here"""
-        self._cpo_store = cpo.pofile(encoding=self._encoding, noheader=True)
+    def serialize(self, out):
+        """Write content to file"""
+        self._cpo_store = cpo.pofile(encoding=self.encoding, noheader=True)
         try:
             self._build_cpo_from_self()
         except UnicodeEncodeError as e:
-            self._encoding = "utf-8"
+            self.encoding = "utf-8"
             self.updateheader(add=True, Content_Type="text/plain; charset=UTF-8")
             self._build_cpo_from_self()
-        output = str(self._cpo_store)
+        self._cpo_store.serialize(out)
         del self._cpo_store
-        return output
