@@ -1,7 +1,7 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2002-2007 Zuza Software Foundation
+# Copyright 2016 F Wolff
 #
 # This file is part of translate.
 #
@@ -19,6 +19,7 @@
 # along with this program; if not, see <http://www.gnu.org/licenses/>.
 
 import re
+import six
 
 
 """
@@ -34,17 +35,22 @@ From the GNU gettext manual:
      msgstr TRANSLATED-STRING
 """
 
-isspace = str.isspace
-find = str.find
-rfind = str.rfind
-startswith = str.startswith
+SINGLE_BYTE_ENCODING = 'iso-8859-1'
+isspace = six.text_type.isspace
+find = six.text_type.find
+rfind = six.text_type.rfind
+startswith = six.text_type.startswith
 append = list.append
-decode = str.decode
+decode = bytes.decode
 
 
 class ParseState(object):
 
-    def __init__(self, input_iterator, UnitClass, encoding=None):
+    def __init__(self, input_iterator, UnitClass, encoding=SINGLE_BYTE_ENCODING):
+        # A single-byte encoding is first defined to be able to read the header
+        # without risking UnicodeDecodeErrors. As soon as the header is parsed,
+        # the encoding defined in the header is used for re-encoding the header
+        # and for decoding all further strings.
         self._input_iterator = input_iterator
         self.next_line = ''
         self.eof = False
@@ -63,12 +69,15 @@ class ParseState(object):
         if self.eof:
             return current
         try:
-            self.next_line = self._input_iterator.next()
-            while not self.eof and isspace(self.next_line):
-                self.next_line = self._input_iterator.next()
+            self.next_line = next(self._input_iterator)
+            while not self.eof and self.next_line.isspace():
+                self.next_line = next(self._input_iterator)
         except StopIteration:
-            self.next_line = ''
+            self.next_line = u''
             self.eof = True
+        else:
+            if isinstance(self.next_line, bytes) and self.encoding is not None:
+                self.next_line = decode(self.next_line, self.encoding)
         return current
 
     def new_input(self, _input):
@@ -76,9 +85,10 @@ class ParseState(object):
 
 
 def read_prevmsgid_lines(parse_state):
-    """Read all the lines belonging starting with #|. These lines contain
-    the previous msgid and msgctxt info. We strip away the leading '#| '
-    and read until we stop seeing #|."""
+    """Read all the lines belonging starting with #|. These lines contain the
+    previous msgid and msgctxt info. We strip away the leading '#| ' and read
+    until we stop seeing #|.
+    """
     prevmsgid_lines = []
     next_line = parse_state.next_line
     while startswith(next_line, '#| ') or startswith(next_line, '| '):
@@ -110,7 +120,7 @@ def parse_comment(parse_state, unit):
     if len(next_line) > 0 and next_line[0] in ('#', '|'):
         next_char = next_line[1]
         if next_char == '.':
-            append(unit.automaticcomments, parse_state.decode(next_line))
+            append(unit.automaticcomments, next_line)
         elif next_line[0] == '|' or next_char == '|':
             # Read all the lines starting with #|
             prevmsgid_lines = read_prevmsgid_lines(parse_state)
@@ -124,15 +134,15 @@ def parse_comment(parse_state, unit):
             parse_prev_msgid_plural(ps, unit)
             return parse_state.next_line
         elif next_char == ':':
-            append(unit.sourcecomments, parse_state.decode(next_line))
+            append(unit.sourcecomments, next_line)
         elif next_char == ',':
-            append(unit.typecomments, parse_state.decode(next_line))
+            append(unit.typecomments, next_line)
         elif next_char == '~':
             # Special case: we refuse to parse obsoletes: they are done
             # elsewhere to ensure we reuse the normal unit parsing code
             return None
         else:
-            append(unit.othercomments, parse_state.decode(next_line))
+            append(unit.othercomments, next_line)
         return parse_state.read_line()
     else:
         return None
@@ -192,7 +202,7 @@ def parse_quoted(parse_state, start_pos=0):
 
 def parse_msg_comment(parse_state, msg_comment_list, string):
     while string is not None:
-        append(msg_comment_list, parse_state.decode(string))
+        append(msg_comment_list, string)
         if find(string, '\\n') > -1:
             return parse_quoted(parse_state)
         string = parse_quoted(parse_state)
@@ -202,16 +212,14 @@ def parse_msg_comment(parse_state, msg_comment_list, string):
 def parse_multiple_quoted(parse_state, msg_list, msg_comment_list, first_start_pos=0):
     string = parse_quoted(parse_state, first_start_pos)
     while string is not None:
-        if not startswith(string, '"_:'):
-            append(msg_list, parse_state.decode(string))
+        if msg_comment_list is None or not startswith(string, '"_:'):
+            append(msg_list, string)
             string = parse_quoted(parse_state)
         else:
             string = parse_msg_comment(parse_state, msg_comment_list, string)
 
 
 def parse_message(parse_state, start_of_string, start_of_string_len, msg_list, msg_comment_list=None):
-    if msg_comment_list is None:
-        msg_comment_list = []
     if startswith(parse_state.next_line, start_of_string):
         return parse_multiple_quoted(parse_state, msg_list, msg_comment_list, start_of_string_len)
 
@@ -235,6 +243,7 @@ def parse_msgid_plural(parse_state, unit):
     parse_message(parse_state, 'msgid_plural', 12, unit.msgid_plural, unit.msgid_pluralcomments)
     return len(unit.msgid_plural) > 0 or len(unit.msgid_pluralcomments) > 0
 
+
 MSGSTR_ARRAY_ENTRY_LEN = len('msgstr[')
 
 
@@ -308,7 +317,7 @@ def parse_unit(parse_state, unit=None):
 
 def set_encoding(parse_state, store, unit):
     charset = None
-    if isinstance(unit.msgstr, list) and len(unit.msgstr) > 0 and isinstance(unit.msgstr[0], str):
+    if isinstance(unit.msgstr, list) and len(unit.msgstr) and isinstance(unit.msgstr[0], six.string_types):
         charset = re.search("charset=([^\\s\\\\n]+)", "".join(unit.msgstr))
     if charset:
         encoding = charset.group(1)
@@ -322,10 +331,15 @@ def set_encoding(parse_state, store, unit):
 
 
 def decode_list(lst, decode):
-    return [decode(item) for item in lst]
+    return [decode(item.encode(SINGLE_BYTE_ENCODING)) for item in lst]
 
 
 def decode_header(unit, decode):
+    """
+    The header has been arbitrarily decoded with a single-byte encoding. We
+    re-encode it to decode values with the proper encoding defined in the header
+    (using decode_list above).
+    """
     for attr in ('msgctxt', 'msgid', 'msgid_pluralcomments',
                  'msgid_plural', 'msgstr',
                  'othercomments', 'automaticcomments', 'sourcecomments',
@@ -343,6 +357,12 @@ def parse_header(parse_state, store):
         return None
     set_encoding(parse_state, store, first_unit)
     decode_header(first_unit, parse_state.decode)
+    # Fix encoding of next line in parser
+    # It was originally parsed with  SINGLE_BYTE_ENCODING
+    # but we need to convert it to actual encoding
+    parse_state.next_line = parse_state.decode(
+        parse_state.next_line.encode(SINGLE_BYTE_ENCODING)
+    )
     return first_unit
 
 
