@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2010 Zuza Software Foundation
@@ -20,19 +19,22 @@
 
 import os
 
+import six
+
 from translate.convert import factory as convert_factory
 from translate.storage.projstore import ProjectStore
 
 
-__all__ = ['Project']
+__all__ = ('Project', )
 
 
 def split_extensions(filename):
-    """Split the given filename into a name and extensions part.
-        The extensions part is defined by any sequence of extensions, where an
-        extension is a 3-letter, .-separated string or one of "po" or
-        "properties". If the file name consists entirely out of extensions, the
-        first part is assumed to be the file name and the rest extensions."""
+    """Split the given filename into a name and extensions part.  The
+    extensions part is defined by any sequence of extensions, where an
+    extension is a 3-letter, .-separated string or one of "po" or "properties".
+    If the file name consists entirely out of extensions, the first part is
+    assumed to be the file name and the rest extensions.
+    """
     # FIXME: Isn't there a better place for this function?
     # XXX: Make sure that all extensions supported in translate.convert.factory
     #      that are not 3 letters long are added to the first "if" statement in
@@ -54,7 +56,8 @@ def split_extensions(filename):
 
 class Project(object):
     """Manages a project store as well as the processes involved in a project
-        workflow."""
+    workflow.
+    """
 
     # INITIALIZERS #
     def __init__(self, projstore=None):
@@ -73,7 +76,8 @@ class Project(object):
 
     def add_source_convert(self, srcfile, src_fname=None, convert_options=None, extension=None):
         """Convenience method that calls :meth:`~Project.add_source` and
-        :meth:`~Project.convert_forward` and returns the results from both."""
+        :meth:`~Project.convert_forward` and returns the results from both.
+        """
         srcfile, srcfname = self.add_source(srcfile, src_fname)
         transfile, transfname = self.convert_forward(srcfname, convert_options=convert_options)
         return srcfile, srcfname, transfile, transfname
@@ -90,10 +94,11 @@ class Project(object):
 
         :type  input_fname: basestring
         :param input_fname: The project name of the file to convert
-        :type  convert_options: dict (optional)
+        :type  convert_options: Dictionary (optional)
         :param convert_options: Passed as-is to
                                 :meth:`translate.convert.factory.convert`.
-        :returns 2-tuple: the converted file object and its project name."""
+        :returns 2-tuple: the converted file object and its project name.
+        """
         inputfile = self.get_file(input_fname)
         input_type = self.store.get_filename_type(input_fname)
 
@@ -101,7 +106,7 @@ class Project(object):
             raise ValueError('Cannot convert a target document further: %s' % (input_fname))
 
         templ_fname = None
-        if isinstance(template, basestring):
+        if isinstance(template, six.string_types):
             template, templ_fname = self.get_file(template)
 
         if template and not templ_fname:
@@ -179,13 +184,14 @@ class Project(object):
         return outputfile, output_fname
 
     def export_file(self, fname, destfname):
-        """Export the file with the specified filename to the given destination.
-            This method will raise
-            :exc:`~translate.storage.projstore.FileNotInProjectError`
-            via the call to
-            :meth:`~translate.storage.projstore.ProjectStore.get_file`
-            if *fname* is not found in the project."""
-        open(destfname, 'w').write(self.store.get_file(fname).read())
+        """Export the file with the specified filename to the given
+        destination.  This method will raise
+        :exc:`~translate.storage.projstore.FileNotInProjectError` via the call
+        to :meth:`~translate.storage.projstore.ProjectStore.get_file` if
+        *fname* is not found in the project.
+        """
+        with open(destfname, 'wb') as fp:
+            fp.write(self.store.get_file(fname).read())
 
     def get_file(self, fname):
         """Proxy for ``self.store.get_file()``."""
