@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2002-2009 Zuza Software Foundation
@@ -24,16 +23,20 @@ files (pofile).
 """
 
 import copy
+import logging
 import re
+import six
 import textwrap
-from cStringIO import StringIO
+from io import BytesIO
 
 from translate.lang import data
 from translate.misc import quote
 from translate.misc.deprecation import deprecated
 from translate.misc.multistring import multistring
-from translate.storage import base, pocommon, poparser
-from translate.storage.pocommon import encodingToUse
+from translate.storage import pocommon, poparser
+
+
+logger = logging.getLogger(__name__)
 
 
 lsep = "\n#: "
@@ -53,8 +56,7 @@ def escapeforpo(line):
     special_locations = []
     for special_key in po_escape_map:
         special_locations.extend(quote.find_all(line, special_key))
-    special_locations = dict.fromkeys(special_locations).keys()
-    special_locations.sort()
+    special_locations = sorted(dict.fromkeys(special_locations).keys())
     escaped_line = ""
     last_location = 0
     for location in special_locations:
@@ -70,11 +72,8 @@ def unescapehandler(escape):
 
 
 wrapper = textwrap.TextWrapper(
-        width=77,
-        replace_whitespace=False,
-        expand_tabs=False,
-        drop_whitespace=False
-)
+    width=77, replace_whitespace=False, expand_tabs=False,
+    drop_whitespace=False)
 wrapper.wordsep_re = re.compile(
     r'(\s+|'                                  # any whitespace
     r'\w*\\.|'                                # any escape should not be split
@@ -85,8 +84,8 @@ wrapper.wordsep_re_uni = re.compile(wrapper.wordsep_re.pattern, re.UNICODE)
 
 
 def quoteforpo(text):
-    """Quotes the given text for a PO file, returning quoted and
-    escaped lines"""
+    """Quotes the given text for a PO file, returning quoted and escaped lines
+    """
     if text is None:
         return []
     text = escapeforpo(text)
@@ -169,6 +168,7 @@ def extractstr(string):
         return string[left:] + '"'
 
 
+@six.python_2_unicode_compatible
 class pounit(pocommon.pounit):
     # othercomments = []      #   # this is another comment
     # automaticcomments = []  #   #. comment extracted from the source code
@@ -186,8 +186,7 @@ class pounit(pocommon.pounit):
     # fashion
     __shallow__ = ['_store']
 
-    def __init__(self, source=None, encoding="UTF-8"):
-        self._encoding = encodingToUse(encoding)
+    def __init__(self, source=None, **kwargs):
         self.obsolete = False
         self._initallcomments(blankall=True)
         self.prev_msgctxt = []
@@ -223,14 +222,15 @@ class pounit(pocommon.pounit):
         singular = unquotefrompo(msgid)
         if self.hasplural():
             pluralform = unquotefrompo(msgid_plural)
-            return multistring([singular, pluralform], self._encoding)
+            return multistring([singular, pluralform])
         return singular
 
     def _set_source_vars(self, source):
         msgid = None
         msgid_plural = None
-        if isinstance(source, str):
-            source = source.decode(self._encoding)
+        if isinstance(source, bytes):
+            # Guessing utf-8, non-ascii encoded content should not reach this point
+            source = source.decode('utf-8')
         if isinstance(source, multistring):
             source = source.strings
         if isinstance(source, list):
@@ -272,19 +272,20 @@ class pounit(pocommon.pounit):
     def gettarget(self):
         """Returns the unescaped msgstr"""
         if isinstance(self.msgstr, dict):
-            return multistring(map(unquotefrompo, self.msgstr.values()), self._encoding)
+            return multistring(list(map(unquotefrompo, self.msgstr.values())))
         else:
             return unquotefrompo(self.msgstr)
 
     def settarget(self, target):
         """Sets the msgstr to the given (unescaped) value"""
         self._rich_target = None
-        if isinstance(target, str):
-            target = target.decode(self._encoding)
+        if isinstance(target, bytes):
+            # Guessing utf-8, non-ascii encoded content should not reach this point
+            target = target.decode('utf-8')
         if self.hasplural():
             if isinstance(target, multistring):
                 target = target.strings
-            elif isinstance(target, basestring):
+            elif isinstance(target, six.string_types):
                 target = [target]
         elif isinstance(target, (dict, list)):
             if len(target) == 1:
@@ -297,7 +298,7 @@ class pounit(pocommon.pounit):
         if isinstance(target, list):
             self.msgstr = dict([(i, quoteforpo(target[i])) for i in range(len(target))])
         elif isinstance(target, dict):
-            self.msgstr = dict([(i, quoteforpo(targetstring)) for i, targetstring in target.iteritems()])
+            self.msgstr = dict([(i, quoteforpo(targetstring)) for i, targetstring in six.iteritems(target)])
         else:
             self.msgstr = quoteforpo(target)
     target = property(gettarget, settarget)
@@ -306,7 +307,8 @@ class pounit(pocommon.pounit):
         """Return a list of alternate units.
 
         Previous msgid and current msgstr is combined to form a single
-        alternative unit."""
+        alternative unit.
+        """
         prev_source = self.prev_source
         if prev_source and self.isfuzzy():
             unit = type(self)(prev_source)
@@ -374,7 +376,7 @@ class pounit(pocommon.pounit):
         # self.__shallow__
         shallow = set(self.__shallow__)
         # Make deep copies of all members which are not in shallow
-        for key, value in self.__dict__.iteritems():
+        for key, value in six.iteritems(self.__dict__):
             if key not in shallow:
                 setattr(new_unit, key, copy.deepcopy(value))
         # Make shallow copies of all members which are in shallow
@@ -397,7 +399,7 @@ class pounit(pocommon.pounit):
 
     def _msgstrlen(self):
         if isinstance(self.msgstr, dict):
-            combinedstr = "\n".join(filter(None, [unquotefrompo(msgstr) for msgstr in self.msgstr.itervalues()]))
+            combinedstr = "\n".join(filter(None, [unquotefrompo(msgstr) for msgstr in six.itervalues(self.msgstr)]))
             return len(combinedstr)
         else:
             return len(unquotefrompo(self.msgstr))
@@ -410,13 +412,13 @@ class pounit(pocommon.pounit):
         """
 
         def mergelists(list1, list2, split=False):
-            #decode where necessary
-            if unicode in [type(item) for item in list2] + [type(item) for item in list1]:
+            # Decode where necessary (either all bytestrings or all unicode)
+            if six.text_type in [type(item) for item in list2] + [type(item) for item in list1]:
                 for position, item in enumerate(list1):
-                    if isinstance(item, str):
+                    if isinstance(item, bytes):
                         list1[position] = item.decode("utf-8")
                 for position, item in enumerate(list2):
-                    if isinstance(item, str):
+                    if isinstance(item, bytes):
                         list2[position] = item.decode("utf-8")
 
             #Determine the newline style of list1
@@ -441,7 +443,7 @@ class pounit(pocommon.pounit):
                 for item in list2:
                     splitlist2.extend(item.split()[1:])
                     prefix = item.split()[0]
-                list1.extend(["%s %s%s" % (prefix, item, lineend) for item in splitlist2 if not item in splitlist1])
+                list1.extend(["%s %s%s" % (prefix, item, lineend) for item in splitlist2 if item not in splitlist1])
             else:
                 #Normal merge, but conform to list1 newline style
                 if list1 != list2:
@@ -484,9 +486,9 @@ class pounit(pocommon.pounit):
         #return (self._msgidlen() == 0) and (self._msgstrlen() > 0) and (len(self.msgidcomments) == 0)
         #rewritten here for performance:
         return (is_null(self.msgid)
-                        and not is_null(self.msgstr)
-                        and self.msgidcomments == []
-                        and is_null(self.msgctxt))
+                and not is_null(self.msgstr)
+                and self.msgidcomments == []
+                and is_null(self.msgctxt))
 
     def isblank(self):
         if self.isheader() or len(self.msgidcomments):
@@ -576,12 +578,11 @@ class pounit(pocommon.pounit):
         return len(self.msgid_plural) > 0
 
     def parse(self, src):
-        return poparser.parse_unit(poparser.ParseState(StringIO(src), pounit), self)
+        return poparser.parse_unit(poparser.ParseState(BytesIO(src), pounit), self)
 
     def _getmsgpartstr(self, partname, partlines, partcomments=""):
         if isinstance(partlines, dict):
-            partkeys = partlines.keys()
-            partkeys.sort()
+            partkeys = sorted(partlines.keys())
             return "".join([self._getmsgpartstr("%s[%d]" % (partname, partkey), partlines[partkey], partcomments) for partkey in partkeys])
         partstr = partname + " "
         partstartline = 0
@@ -621,18 +622,9 @@ class pounit(pocommon.pounit):
             partstr += partline + '\n'
         return partstr
 
-    def _encodeifneccessary(self, output):
-        """Encodes unicode strings and returns other strings unchanged"""
-        if isinstance(output, unicode):
-            encoding = encodingToUse(getattr(self, "_encoding", "UTF-8"))
-            return output.encode(encoding)
-        return output
-
     def __str__(self):
-        """Convert to a string. Double check that unicode is handled
-        somehow here"""
-        output = self._getoutput()
-        return self._encodeifneccessary(output)
+        """Convert to a string."""
+        return self._getoutput()
 
     def _getoutput(self):
         """return this po element as a string"""
@@ -755,24 +747,25 @@ class pounit(pocommon.pounit):
 
 class pofile(pocommon.pofile):
     """A .po file containing various units"""
+
     UnitClass = pounit
 
     def parse(self, input):
         """Parses the given file or file source string."""
-        if True:
-            if hasattr(input, 'name'):
-                self.filename = input.name
-            elif not getattr(self, 'filename', ''):
-                self.filename = ''
-            if isinstance(input, str):
-                input = StringIO(input)
-            # clear units to get rid of automatically generated headers before parsing
-            self.units = []
-            poparser.parse_units(poparser.ParseState(input, pounit), self)
+        if hasattr(input, 'name'):
+            self.filename = input.name
+        elif not getattr(self, 'filename', ''):
+            self.filename = ''
+        if isinstance(input, bytes):
+            input = BytesIO(input)
+        # clear units to get rid of automatically generated headers before parsing
+        self.units = []
+        poparser.parse_units(poparser.ParseState(input, pounit), self)
 
     def removeduplicates(self, duplicatestyle="merge"):
         """Make sure each msgid is unique ; merge comments etc from
-        duplicates into original"""
+        duplicates into original
+        """
         # TODO: can we handle consecutive calls to removeduplicates()? What
         # about files already containing msgctxt? - test
         id_dict = {}
@@ -803,7 +796,13 @@ class pofile(pocommon.pofile):
                         origpo.msgctxt.append('"%s"' % escapeforpo(" ".join(origpo.getlocations())))
                         markedpos.append(thepo)
                     thepo.msgctxt.append('"%s"' % escapeforpo(" ".join(thepo.getlocations())))
-                    uniqueunits.append(thepo)
+                    if not thepo.msgctxt == id_dict[id].msgctxt:
+                        uniqueunits.append(thepo)
+                    else:
+                        logger.warn(
+                            "Duplicate unit found with msgctx of '%s' and source '%s'",
+                            thepo.msgctxt,
+                            thepo.source)
             else:
                 if not id:
                     if duplicatestyle == "merge":
@@ -814,57 +813,43 @@ class pofile(pocommon.pofile):
                 uniqueunits.append(thepo)
         self.units = uniqueunits
 
-    def __str__(self):
-        """Convert to a string. Double check that unicode is handled somehow
-        here"""
-        output = self._getoutput()
-        if isinstance(output, unicode):
-            try:
-                return output.encode(getattr(self, "_encoding", "UTF-8"))
-            except UnicodeEncodeError as e:
-                self.updateheader(add=True, Content_Type="text/plain; charset=UTF-8")
-                self._encoding = "UTF-8"
-                for unit in self.units:
-                    unit._encoding = "UTF-8"
-                return self._getoutput().encode("UTF-8")
-
-        return output
-
-    def _getoutput(self):
-        """convert the units back to lines"""
-        lines = []
-        for unit in self.units:
-            unitsrc = unit._getoutput() + u"\n"
-            lines.append(unitsrc)
-        lines = u"".join(lines).rstrip()
-        #After the last pounit we will have \n\n and we only want to end in \n:
-        if lines:
-            lines += u"\n"
-        return lines
+    def serialize(self, out):
+        """Write to file"""
+        at_start = True
+        try:
+            for unit in self.units:
+                if not at_start:
+                    out.write(b'\n')
+                else:
+                    at_start = False
+                out.write(unit._getoutput().encode(self.encoding))
+        except UnicodeEncodeError as e:
+            if self.encoding == 'utf-8':
+                raise
+            self.updateheader(add=True, Content_Type="text/plain; charset=UTF-8")
+            self.encoding = "utf-8"
+            out.seek(0)
+            self.serialize(out)
 
     def encode(self, lines):
-        """encode any unicode strings in lines in self._encoding"""
+        """encode any unicode strings in lines in self.encoding"""
         newlines = []
-        encoding = self._encoding
-        if encoding is None or encoding.lower() == "charset":
-            encoding = 'UTF-8'
         for line in lines:
-            if isinstance(line, unicode):
-                line = line.encode(encoding)
+            if isinstance(line, six.text_type):
+                line = line.encode(self.encoding)
             newlines.append(line)
         return newlines
 
     def decode(self, lines):
-        """decode any non-unicode strings in lines with self._encoding"""
+        """decode any non-unicode strings in lines with self.encoding"""
         newlines = []
         for line in lines:
-            if (isinstance(line, str) and self._encoding is not None and
-                self._encoding.lower() != "charset"):
+            if isinstance(line, bytes):
                 try:
-                    line = line.decode(self._encoding)
-                except UnicodeError as e:
-                    raise UnicodeError("Error decoding line with encoding %r: %s. Line is %r" %
-                                       (self._encoding, e, line))
+                    line = line.decode(self.encoding)
+                except UnicodeDecodeError as e:
+                    raise UnicodeDecodeError("Error decoding line with encoding %r: %s. Line is %r" %
+                                             (self.encoding, e, line))
             newlines.append(line)
         return newlines
 
