@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2015 Zuza Software Foundation
@@ -24,9 +23,11 @@
 from lxml import etree
 
 from translate.lang import data
+from translate.misc.xml_helpers import setXMLspace
 from translate.storage import base, lisa
 from translate.storage.placeables import general
 
+
 class RESXUnit(lisa.LISAunit):
     """A single term in the RESX file."""
 
@@ -48,13 +49,13 @@ class RESXUnit(lisa.LISAunit):
         return self.xmlelement.find(self.namespaced(self.languageNode))
 
     def getsource(self):
-        return self.gettarget()
+        return self.target
 
     def settarget(self, text, lang='xx', append=False):
         # Firstly deal with reinitialising to None or setting to identical
         # string.
         self._rich_target = None
-        if self.gettarget() == text:
+        if self.target == text:
             return
         strings = []
         if isinstance(text, list):
@@ -64,7 +65,10 @@ class RESXUnit(lisa.LISAunit):
         targetnode = self._gettargetnode()
         targetnode.clear()
         targetnode.text = data.forceunicode(text) or u""
-        targetnode.tail = u"\n    "
+        # Assume no <comment> follows; allow the </data> element
+        # to be indented with 2 spaces (same level as the opening
+        # <data> element before <value>)
+        targetnode.tail = u"\n  "
 
     def gettarget(self, lang=None):
         targetnode = self._gettargetnode()
@@ -77,7 +81,7 @@ class RESXUnit(lisa.LISAunit):
 
     def addnote(self, text, origin=None, position="append"):
         """Add a note specifically in the appropriate "comment" tag"""
-        if isinstance(text, str):
+        if isinstance(text, bytes):
             text = text.decode("utf-8")
         current_notes = self.getnotes(origin)
         self.removenotes(origin)
@@ -91,6 +95,14 @@ class RESXUnit(lisa.LISAunit):
                 note.text = "\n".join(filter(None, [current_notes, text_stripped]))
         else:
             note.text = text_stripped
+        if note.text:
+            # Correct the indent of <comment> by updating the tail of
+            # the preceding <value> element
+            targetnode = self._gettargetnode()
+            targetnode.tail = u"\n    "
+        # This also requires putting a matching tail on the <comment>,
+        # otherwise the </data> would yet again be misaligned
+        note.tail = u"\n  "
 
     def getnotes(self, origin=None):
         comments = []
@@ -101,7 +113,7 @@ class RESXUnit(lisa.LISAunit):
 
     def removenotes(self, origin=None):
         note = self.xmlelement.find(self.namespaced("comment"))
-        if not note is None:
+        if note is not None:
             self.xmlelement.remove(note)
 
     def setid(self, value):
@@ -124,6 +136,7 @@ class RESXUnit(lisa.LISAunit):
 
 class RESXFile(lisa.LISAfile):
     """Class representing a RESX file store."""
+
     UnitClass = RESXUnit
     Name = ".NET RESX File"
     Mimetypes = ["text/microsoft-resx"]
@@ -185,6 +198,12 @@ class RESXFile(lisa.LISAfile):
   <resheader name="version">
     <value>2.0</value>
   </resheader>
+  <resheader name="reader">
+    <value>System.Resources.ResXResourceReader, System.Windows.Forms, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089</value>
+  </resheader>
+  <resheader name="writer">
+    <value>System.Resources.ResXResourceWriter, System.Windows.Forms, Version=4.0.0.0, Culture=neutral, PublicKeyToken=b77a5c561934e089</value>
+  </resheader>
 </root>
 '''
     namespace = ''
@@ -202,7 +221,7 @@ class RESXFile(lisa.LISAfile):
     def addunit(self, unit, new=True):
         """Adds the given unit to the body node."""
         super(RESXFile, self).addunit(unit, new)
-        lisa.setXMLspace(unit.xmlelement, "preserve")
+        setXMLspace(unit.xmlelement, "preserve")
         if unit.getid() is None:
             self._messagenum += 1
             unit.setid(u"%s" % unit.source.strip(' '))
