@@ -1,6 +1,7 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
+from __future__ import print_function
 
+import six
 from pytest import raises
 
 from translate.misc import wStringIO
@@ -33,25 +34,25 @@ class TestHelpers():
         # Newline handling
         assert pypo.quoteforpo("One\nTwo\n") == ['""', '"One\\n"', '"Two\\n"']
         # First line wrapping
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A ver") == \
-                             ['"A very long sentence. A very long sentence. A very long sentence. A ver"']
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very") == \
-                              ['""',
-                               '"A very long sentence. A very long sentence. A very long sentence. A very"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A ver") == [
+            '"A very long sentence. A very long sentence. A very long sentence. A ver"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very") == [
+            '""',
+            '"A very long sentence. A very long sentence. A very long sentence. A very"']
         # Long line with a newline
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very lon\n") == \
-                             ['""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"lon\\n"']
-        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very 123\n") == \
-                             ['""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"123\\n"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very lon\n") == [
+            '""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"lon\\n"']
+        assert pypo.quoteforpo("A very long sentence. A very long sentence. A very long sentence. A very 123\n") == [
+            '""', '"A very long sentence. A very long sentence. A very long sentence. A very "', '"123\\n"']
         # Special 77 char failure.
-        assert pypo.quoteforpo("Ukuba uyayiqonda into eyenzekayo, \nungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. \n<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu \nobhucabhuca ukudibanisa kwakho.</b>") == \
-                             ['""',
-                              '"Ukuba uyayiqonda into eyenzekayo, \\n"',
-                              '"ungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. "',
-                              '"\\n"',
-                              '"<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu "',
-                              '"\\n"',
-                              '"obhucabhuca ukudibanisa kwakho.</b>"']
+        assert pypo.quoteforpo("Ukuba uyayiqonda into eyenzekayo, \nungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. \n<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu \nobhucabhuca ukudibanisa kwakho.</b>") == [
+            '""',
+            '"Ukuba uyayiqonda into eyenzekayo, \\n"',
+            '"ungaxelela i-&brandShortName; ukuba iqalise ukuthemba ufaniso lwale sayithi. "',
+            '"\\n"',
+            '"<b>Nokuba uyayithemba isayithi, le mposiso isenokuthetha ukuba   kukho umntu "',
+            '"\\n"',
+            '"obhucabhuca ukudibanisa kwakho.</b>"']
 
     def test_quoteforpo_escaped_quotes(self):
         """Ensure that we don't break \" in two when wrapping
@@ -206,7 +207,7 @@ class TestPYPOFile(test_po.TestPOFile):
         thepo = pofile.units[0]
         thepo.msgidcomments.append('"_: first comment\\n"')
         thepo.msgidcomments.append('"_: second comment\\n"')
-        regenposource = str(pofile)
+        regenposource = bytes(pofile).decode('utf-8')
         assert regenposource.count("_:") == 1
 
     def test_merge_duplicates_msgctxt(self):
@@ -237,19 +238,19 @@ class TestPYPOFile(test_po.TestPOFile):
         posource = u'''#: nb\nmsgid "Norwegian Bokm\xe5l"\nmsgstr ""\n'''
         pofile = self.StoreClass(wStringIO.StringIO(posource.encode("UTF-8")), encoding="UTF-8")
         assert len(pofile.units) == 1
-        print(str(pofile))
+        print(bytes(pofile))
         thepo = pofile.units[0]
-        assert str(thepo) == posource.encode("UTF-8")
-        # extra test: what if we set the msgid to a unicode? this happens in prop2po etc
-        thepo.source = u"Norwegian Bokm\xe5l"
-        assert str(thepo) == posource.encode("UTF-8")
+        # On Python 2, str() returns bytestrings while on Python 3, str() returns unicode
+        assert str(thepo) == posource if six.PY3 else posource.encode("UTF-8")
+        assert six.text_type(thepo) == posource
         # Now if we set the msgstr to Unicode
         # this is an escaped half character (1/2)
-        halfstr = "\xbd ...".decode("latin-1")
+        halfstr = b"\xbd ...".decode("latin-1")
         thepo.target = halfstr
-        assert halfstr in str(thepo).decode("UTF-8")
+        assert halfstr in six.text_type(thepo)
         thepo.target = halfstr.encode("UTF-8")
-        assert halfstr.encode("UTF-8") in str(thepo)
+        if six.PY2:
+            assert halfstr.encode("UTF-8") in str(thepo)
 
     def test_posections(self):
         """checks the content of all the expected sections of a PO message"""
@@ -257,7 +258,7 @@ class TestPYPOFile(test_po.TestPOFile):
         pofile = self.poparse(posource)
         print(pofile)
         assert len(pofile.units) == 1
-        assert str(pofile) == posource
+        assert bytes(pofile).decode('utf-8') == posource
         assert pofile.units[0].othercomments == ["# other comment\n"]
         assert pofile.units[0].automaticcomments == ["#. automatic comment\n"]
         assert pofile.units[0].sourcecomments == ["#: source comment\n"]
@@ -267,7 +268,7 @@ class TestPYPOFile(test_po.TestPOFile):
         """tests behaviour of unassociated comments."""
         oldsource = '# old lonesome comment\n\nmsgid "one"\nmsgstr "een"\n'
         oldfile = self.poparse(oldsource)
-        print(str(oldfile))
+        print(bytes(oldfile))
         assert len(oldfile.units) == 1
 
     def test_prevmsgid_parse(self):
@@ -321,4 +322,4 @@ msgstr[1] "toetse"
         assert pofile.units[4].prev_msgctxt == [u'"context 2"']
         assert pofile.units[4].prev_source == multistring([u"tast", u"tasts"])
 
-        assert str(pofile) == posource
+        assert bytes(pofile).decode('utf-8') == posource
