@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+from py.test import deprecated_call
 
 from translate.misc import wStringIO
 from translate.storage import test_monolingual, txt
@@ -19,7 +19,7 @@ class TestTxtFile(test_monolingual.TestMonolingualStore):
 
     def txtregen(self, txtsource):
         """helper that converts txt source to txtfile object and back"""
-        return str(self.txtparse(txtsource))
+        return bytes(self.txtparse(txtsource)).decode('utf-8')
 
     def test_simpleblock(self):
         """checks that a simple txt block is parsed correctly"""
@@ -35,7 +35,14 @@ class TestTxtFile(test_monolingual.TestMonolingualStore):
         txtfile = self.txtparse(txtsource)
         assert len(txtfile.units) == 3
         print(txtsource)
-        print(str(txtfile))
+        print(bytes(txtfile))
         print("*%s*" % txtfile.units[0])
-        assert str(txtfile) == txtsource
+        assert bytes(txtfile).decode('utf-8') == txtsource
         assert self.txtregen(txtsource) == txtsource
+
+    def test_getoutput(self):
+        # Test deprecated method still works
+        txtsource = 'bananas for sale'
+        txtfile = self.txtparse(txtsource)
+        assert txtfile.getoutput() == bytes(txtfile)
+        deprecated_call(txtfile.getoutput)
