@@ -1,8 +1,9 @@
-#!/usr/bin/env python
-
+import pytest
+import six
 from lxml import etree
 
-from translate.storage import lisa, test_base, xliff
+from translate.misc.xml_helpers import setXMLspace
+from translate.storage import test_base, xliff
 from translate.storage.placeables import StringElem
 from translate.storage.placeables.xliff import G, X
 
@@ -48,6 +49,43 @@ class TestXLIFFUnit(test_base.TestTranslationUnit):
         unit.adderror(errorname='test1', errortext='New error 1.')
         assert unit.geterrors()['test1'] == 'New error 1.'
 
+    def test_accepted_control_chars(self):
+        """Tests we can assign the accepted control chars.
+
+        Source: https://en.wikipedia.org/wiki/Valid_characters_in_XML#XML_1.0
+        """
+        # Unicode Character 'CHARACTER TABULATION' (U+0009)
+        self.unit.target = u'Een\t'
+        assert self.unit.target == u'Een\t'
+        # Unicode Character 'LINE FEED (LF)' (U+000A)
+        self.unit.target = u'Een\n'
+        assert self.unit.target == u'Een\n'
+        # Unicode Character 'CARRIAGE RETURN (CR)' (U+000D)
+        self.unit.target = u'Een\r'
+        assert self.unit.target == u'Een\r'
+
+    def test_unaccepted_control_chars(self):
+        """Tests we cannot assign the unaccepted control chars without escaping.
+
+        Source: https://en.wikipedia.org/wiki/Valid_characters_in_XML#XML_1.0
+        """
+        exc_msg = ("All strings must be XML compatible: Unicode or ASCII, no "
+                   "NULL bytes or control characters")
+        for code in xliff.ASCII_CONTROL_CODES:
+            self.unit.target = u'Een&#x%s;' % code.lstrip('0') or '0'
+            assert self.unit.target == u'Een%s' % six.unichr(int(code, 16))
+            self.unit.target = u'Een%s' % six.unichr(int(code, 16))
+            assert self.unit.target == u'Een%s' % six.unichr(int(code, 16))
+
+    def test_unaccepted_control_chars_escapes_roundtrip(self):
+        """Test control characters go ok on escaping roundtrip."""
+        for code in xliff.ASCII_CONTROL_CODES:
+            special = u'Een%s' % six.unichr(int(code, 16))
+            self.unit.source = special
+            print("unit.source:", repr(self.unit.source))
+            print("special:", repr(special))
+            assert self.unit.source == special
+
 
 class TestXLIFFfile(test_base.TestTranslationStore):
     StoreClass = xliff.xlifffile
@@ -65,8 +103,8 @@ class TestXLIFFfile(test_base.TestTranslationStore):
         assert xlifffile.units == []
         xlifffile.addsourceunit("Bla")
         assert len(xlifffile.units) == 1
-        newfile = xliff.xlifffile.parsestring(str(xlifffile))
-        print(str(xlifffile))
+        newfile = xliff.xlifffile.parsestring(bytes(xlifffile))
+        print(bytes(xlifffile))
         assert len(newfile.units) == 1
         assert newfile.units[0].source == "Bla"
         assert newfile.findunit("Bla").source == "Bla"
@@ -85,7 +123,7 @@ class TestXLIFFfile(test_base.TestTranslationStore):
     </xliff:file>
 </xliff:xliff>'''
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
-        print(str(xlifffile))
+        print(bytes(xlifffile))
         assert xlifffile.units[0].source == "File 1"
 
     def test_rich_source(self):
@@ -169,8 +207,8 @@ class TestXLIFFfile(test_base.TestTranslationStore):
         xlifffile = xliff.xlifffile()
         xliffunit = xlifffile.addsourceunit("Concept")
         xliffunit.source = "Term"
-        newfile = xliff.xlifffile.parsestring(str(xlifffile))
-        print(str(xlifffile))
+        newfile = xliff.xlifffile.parsestring(bytes(xlifffile))
+        print(bytes(xlifffile))
         assert newfile.findunit("Concept") is None
         assert newfile.findunit("Term") is not None
 
@@ -178,20 +216,20 @@ class TestXLIFFfile(test_base.TestTranslationStore):
         xlifffile = xliff.xlifffile()
         xliffunit = xlifffile.addsourceunit("Concept")
         xliffunit.target = "Konsep"
-        newfile = xliff.xlifffile.parsestring(str(xlifffile))
-        print(str(xlifffile))
+        newfile = xliff.xlifffile.parsestring(bytes(xlifffile))
+        print(bytes(xlifffile))
         assert newfile.findunit("Concept").target == "Konsep"
 
     def test_sourcelanguage(self):
         xlifffile = xliff.xlifffile(sourcelanguage="xh")
-        xmltext = str(xlifffile)
+        xmltext = bytes(xlifffile).decode('utf-8')
         print(xmltext)
         assert xmltext.find('source-language="xh"') > 0
         #TODO: test that it also works for new files.
 
     def test_targetlanguage(self):
         xlifffile = xliff.xlifffile(sourcelanguage="zu", targetlanguage="af")
-        xmltext = str(xlifffile)
+        xmltext = bytes(xlifffile).decode('utf-8')
         print(xmltext)
         assert xmltext.find('source-language="zu"') > 0
         assert xmltext.find('target-language="af"') > 0
@@ -200,11 +238,11 @@ class TestXLIFFfile(test_base.TestTranslationStore):
         xlifffile = xliff.xlifffile()
         unit = xlifffile.addsourceunit("Concept")
         # We don't want to add unnecessary notes
-        assert not "note" in str(xlifffile)
+        assert "note" not in bytes(xlifffile).decode('utf-8')
         unit.addnote(None)
-        assert not "note" in str(xlifffile)
+        assert "note" not in bytes(xlifffile).decode('utf-8')
         unit.addnote("")
-        assert not "note" in str(xlifffile)
+        assert "note" not in bytes(xlifffile).decode('utf-8')
 
         unit.addnote("Please buy bread")
         assert unit.getnotes() == "Please buy bread"
@@ -214,7 +252,7 @@ class TestXLIFFfile(test_base.TestTranslationStore):
         unit.addnote("Please buy milk", origin="Mom")
         notenodes = unit.xmlelement.findall(".//%s" % unit.namespaced("note"))
         assert len(notenodes) == 2
-        assert not "from" in notenodes[0].attrib
+        assert "from" not in notenodes[0].attrib
         assert notenodes[1].get("from") == "Mom"
         assert unit.getnotes(origin="Mom") == "Please buy milk"
 
@@ -227,7 +265,7 @@ class TestXLIFFfile(test_base.TestTranslationStore):
 
         assert not unit.getnotes(origin="Bob") == "Please buy bread\nPlease buy milk\nDon't forget the beer"
         assert not notenodes[2].get("from") == "Mom"
-        assert not "from" in notenodes[0].attrib
+        assert "from" not in notenodes[0].attrib
         assert unit.getnotes() == "Please buy bread\nPlease buy milk\nDon't forget the beer"
         assert unit.correctorigin(notenodes[2], "ad")
         assert not unit.correctorigin(notenodes[2], "om")
@@ -267,7 +305,7 @@ class TestXLIFFfile(test_base.TestTranslationStore):
         unit.addalttrans("targetx", sourcetxt="sourcex")
         # test that the source node is before the target node:
         alt = unit.getalttrans()[0]
-        altformat = etree.tostring(alt.xmlelement)
+        altformat = etree.tostring(alt.xmlelement).decode('utf-8')
         print(altformat)
         assert altformat.find("<source") < altformat.find("<target")
 
@@ -303,78 +341,141 @@ class TestXLIFFfile(test_base.TestTranslationStore):
 
     def test_xml_space(self):
         """Test for the correct handling of xml:space attributes."""
-        xlfsource = self.skeleton \
-          % '''<trans-unit id="1" xml:space="preserve">
+        xlfsource = self.skeleton % (
+            '''<trans-unit id="1" xml:space="preserve">
                    <source> File  1 </source>
-               </trans-unit>'''
+               </trans-unit>''')
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
         assert xlifffile.units[0].source == " File  1 "
         root_node = xlifffile.document.getroot()
-        lisa.setXMLspace(root_node, "preserve")
+        setXMLspace(root_node, "preserve")
         assert xlifffile.units[0].source == " File  1 "
-        lisa.setXMLspace(root_node, "default")
+        setXMLspace(root_node, "default")
         assert xlifffile.units[0].source == " File  1 "
 
-        xlfsource = self.skeleton \
-          % '''<trans-unit id="1" xml:space="default">
+        xlfsource = self.skeleton % (
+            '''<trans-unit id="1" xml:space="default">
                    <source> File  1 </source>
-               </trans-unit>'''
+               </trans-unit>''')
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
         assert xlifffile.units[0].source == "File 1"
         root_node = xlifffile.document.getroot()
-        lisa.setXMLspace(root_node, "preserve")
+        setXMLspace(root_node, "preserve")
         assert xlifffile.units[0].source == "File 1"
-        lisa.setXMLspace(root_node, "default")
+        setXMLspace(root_node, "default")
         assert xlifffile.units[0].source == "File 1"
 
-        xlfsource = self.skeleton \
-          % '''<trans-unit id="1">
+        xlfsource = self.skeleton % (
+            '''<trans-unit id="1">
                    <source> File  1 </source>
-               </trans-unit>'''
+               </trans-unit>''')
         # we currently always normalize as default behaviour for xliff
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
         assert xlifffile.units[0].source == "File 1"
         root_node = xlifffile.document.getroot()
-        lisa.setXMLspace(root_node, "preserve")
+        setXMLspace(root_node, "preserve")
         assert xlifffile.units[0].source == "File 1"
-        lisa.setXMLspace(root_node, "default")
+        setXMLspace(root_node, "default")
         assert xlifffile.units[0].source == "File 1"
 
-        xlfsource = self.skeleton \
-          % '''<trans-unit id="1">
+        xlfsource = self.skeleton % (
+            '''<trans-unit id="1">
                    <source> File  1
 </source>
-               </trans-unit>'''
+               </trans-unit>''')
         # we currently always normalize as default behaviour for xliff
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
         assert xlifffile.units[0].source == "File 1"
         root_node = xlifffile.document.getroot()
-        lisa.setXMLspace(root_node, "preserve")
+        setXMLspace(root_node, "preserve")
         assert xlifffile.units[0].source == "File 1"
-        lisa.setXMLspace(root_node, "default")
+        setXMLspace(root_node, "default")
         assert xlifffile.units[0].source == "File 1"
 
     def test_parsing(self):
         xlfsource = self.skeleton \
-          % '''<trans-unit id="1" xml:space="preserve">
-                   <source>File</source>
-                   <target/>
-               </trans-unit>'''
+            % '''<trans-unit id="1" xml:space="preserve">
+                     <source>File</source>
+                     <target/>
+                 </trans-unit>'''
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
         assert xlifffile.units[0].istranslatable()
 
         xlfsource = self.skeleton \
-          % '''<trans-unit id="1" xml:space="preserve" translate="no">
-                   <source>File</source>
-                   <target/>
-               </trans-unit>'''
+            % '''<trans-unit id="1" xml:space="preserve" translate="no">
+                     <source>File</source>
+                     <target/>
+                 </trans-unit>'''
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
         assert not xlifffile.units[0].istranslatable()
 
         xlfsource = self.skeleton \
-          % '''<trans-unit id="1" xml:space="preserve" translate="yes">
-                   <source>File</source>
-                   <target/>
-               </trans-unit>'''
+            % '''<trans-unit id="1" xml:space="preserve" translate="yes">
+                     <source>File</source>
+                     <target/>
+                 </trans-unit>'''
         xlifffile = xliff.xlifffile.parsestring(xlfsource)
         assert xlifffile.units[0].istranslatable()
+
+    def test_entities(self):
+        xlfsource = '''<?xml version="1.0" encoding="utf-8"?>
+<!DOCTYPE foo [ <!ELEMENT foo ANY > <!ENTITY xxe SYSTEM "file:///etc/passwd" >]>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+        <file original="doc.txt" source-language="en-US">
+                <body>
+                    <trans-unit id="1" xml:space="preserve" translate="yes">
+                        <source>&xxe;</source>
+                        <target/>
+                    </trans-unit>
+                    <trans-unit id="2" xml:space="preserve" translate="yes">
+                        <source>&amp;</source>
+                        <target/>
+                    </trans-unit>
+                </body>
+        </file>
+</xliff>'''
+        xlifffile = xliff.xlifffile.parsestring(xlfsource)
+        assert xlifffile.units[0].istranslatable()
+        assert xlifffile.units[0].source == ''
+        assert xlifffile.units[1].istranslatable()
+        assert xlifffile.units[1].source == '&'
+
+    def test_multiple_filenodes(self):
+        xlfsource = '''<?xml version="1.0" encoding="utf-8"?>
+<xliff version="1.1" xmlns="urn:oasis:names:tc:xliff:document:1.1">
+  <file original="file0" source-language="en" datatype="plaintext">
+    <body>
+      <trans-unit id="hello" approved="yes">
+        <source>Hello</source>
+      </trans-unit>
+    </body>
+  </file>
+  <file original="file1" source-language="en" datatype="plaintext">
+    <body>
+      <trans-unit id="world" approved="yes">
+        <source>World</source>
+      </trans-unit>
+    </body>
+  </file>
+</xliff>'''
+        xfile = xliff.xlifffile.parsestring(xlfsource)
+        assert len(xfile.units) == 2
+        assert xfile.units[0].getid() == "file0\x04hello"
+        assert xfile.units[1].getid() == "file1\x04world"
+        xunit = xliff.xlifffile.UnitClass(source="goodbye")
+        xunit.setid("file2\x04goodbye")
+        xfile.addunit(xunit)
+        assert xfile.units[2].getid() == "file2\x04goodbye"
+        # if there is no file set it will use the active context
+        xunit = xliff.xlifffile.UnitClass(source="lastfile")
+        xunit.setid("lastfile")
+        xfile.addunit(xunit)
+        assert xfile.units[3].getid() == "file2\x04lastfile"
+        newxfile = xliff.xlifffile()
+        newxfile.addunit(xfile.units[0])
+        newxfile.addunit(xfile.units[1])
+        assert newxfile.units[0].getid() == "file0\x04hello"
+        assert newxfile.units[1].getid() == "file1\x04world"
+        assert newxfile.getfilenode("file0")
+        assert newxfile.getfilenode("file1")
+        assert not newxfile.getfilenode("foo")
