@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright 2007 Zuza Software Foundation
@@ -28,7 +27,9 @@ Supported formats are
 """
 
 import re
+import six
 
+from translate.misc.deprecation import deprecated
 from translate.storage import base
 
 
@@ -50,12 +51,12 @@ flavours = {
 }
 
 
+@six.python_2_unicode_compatible
 class TxtUnit(base.TranslationUnit):
     """This class represents a block of text from a text file"""
 
-    def __init__(self, source="", encoding="utf-8"):
+    def __init__(self, source="", **kwargs):
         """Construct the txtunit"""
-        self.encoding = encoding
         super(TxtUnit, self).__init__(source)
         self.source = source
         self.pretext = ""
@@ -64,16 +65,11 @@ class TxtUnit(base.TranslationUnit):
 
     def __str__(self):
         """Convert a txt unit to a string"""
-        string = u"".join([self.pretext, self.source, self.posttext])
-        if isinstance(string, unicode):
-            return string.encode(self.encoding)
-        return string
+        return u"".join([self.pretext, self.source, self.posttext])
 
     # Note that source and target are equivalent for monolingual units
     def setsource(self, source):
         """Sets the definition to the quoted value of source"""
-        if isinstance(source, str):
-            source = source.decode(self.encoding)
         self._rich_source = None
         self._source = source
 
@@ -101,13 +97,13 @@ class TxtUnit(base.TranslationUnit):
 
 class TxtFile(base.TranslationStore):
     """This class represents a text file, made up of txtunits"""
+
     UnitClass = TxtUnit
 
-    def __init__(self, inputfile=None, flavour=None, encoding="utf-8"):
-        base.TranslationStore.__init__(self, unitclass=self.UnitClass)
+    def __init__(self, inputfile=None, flavour=None, **kwargs):
+        super(TxtFile, self).__init__(**kwargs)
         self.filename = getattr(inputfile, 'name', '')
         self.flavour = flavours.get(flavour, [])
-        self.encoding = encoding
         if inputfile is not None:
             txtsrc = inputfile.readlines()
             self.parse(txtsrc)
@@ -115,13 +111,14 @@ class TxtFile(base.TranslationStore):
     def parse(self, lines):
         """Read in text lines and create txtunits from the blocks of text"""
         block = []
-        startline = 0
+        current_line = 0
         pretext = ""
         posttext = ""
         if not isinstance(lines, list):
-            lines = lines.split("\n")
-        for linenum in range(len(lines)):
-            line = lines[linenum].rstrip("\r\n")
+            lines = lines.split(b"\n")
+        for linenum, line in enumerate(lines):
+            current_line = linenum + 1
+            line = line.decode(self.encoding).rstrip("\r\n")
             for rule, prere, postre in self.flavour:
                 match = prere.match(line)
                 if match:
@@ -137,28 +134,25 @@ class TxtFile(base.TranslationStore):
                 isbreak = not line.strip()
             if isbreak and block:
                 unit = self.addsourceunit("\n".join(block))
-                unit.addlocation("%s:%d" % (self.filename, startline + 1))
+                unit.addlocation("%s:%d" % (self.filename, current_line))
                 unit.pretext = pretext
                 unit.posttext = posttext
                 pretext = ""
                 posttext = ""
                 block = []
             elif not isbreak:
-                if not block:
-                    startline = linenum
                 block.append(line)
         if block:
             unit = self.addsourceunit("\n".join(block))
-            unit.addlocation("%s:%d" % (self.filename, startline + 1))
+            unit.addlocation("%s:%d" % (self.filename, current_line))
 
-    def __str__(self):
-        source = self.getoutput()
-        if isinstance(source, unicode):
-            return source.encode(getattr(self, "encoding", "UTF-8"))
-        return source
+    def serialize(self, out):
+        for idx, unit in enumerate(self.units):
+            if idx > 0:
+                out.write(b'\n\n')
+            out.write(six.text_type(unit).encode(self.encoding))
 
+    # Deprecated on 2.0.0
+    @deprecated("Use bytes(TxtFile) instead")
     def getoutput(self):
-        """Convert the units back to blocks"""
-        blocks = [str(unit) for unit in self.units]
-        string = "\n\n".join(blocks)
-        return string
+        return bytes(self)
